// Spatial Index Library
//
// Copyright (C) 2002  Navel Ltd.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Contact information:
//  Mailing address:
//    Marios Hadjieleftheriou
//    University of California, Riverside
//    Department of Computer Science
//    Surge Building, Room 310
//    Riverside, CA 92521
//
//  Email:
//    marioh@cs.ucr.edu

#ifndef __spatialindex_mvrtree_h
#define __spatialindex_mvrtree_h

namespace SpatialIndex
{
	namespace MVRTree
	{
		enum MVRTreeVariant
		{
			RV_LINEAR = 0x0,
			RV_QUADRATIC,
			RV_RSTAR
		};

		enum PersistenObjectIdentifier
		{
			PersistentIndex = 0x1,
			PersistentLeaf = 0x2
		};

		enum RangeQueryType
		{
			ContainmentQuery = 0x1,
			IntersectionQuery = 0x2
		};

		extern ISpatialIndex* returnMVRTree(IStorageManager& sm, Tools::PropertySet& ps)
			throw (std::exception, Tools::Exception);
		extern ISpatialIndex* createNewMVRTree(
			IStorageManager& in,
			double fillFactor,
			unsigned long indexCapacity,
			unsigned long leafCapacity,
			unsigned long dimension,
			MVRTreeVariant rv,
			long& out_indexIdentifier)
			throw (std::exception, Tools::Exception);
		extern ISpatialIndex* loadMVRTree(
			IStorageManager& in,
			long indexIdentifier)
			throw (std::exception, Tools::Exception);
	}
}

#endif /* __spatialindex_mvrtree_h */
