/**
 *  @file       log/streambf.hpp
 *  @author     Taka
 *  @date       23oct97
 *  
 *  @copyright  (c) T. Muraoka 1997 (all rights reserved).
 *              Unauthorized use of this code is prohibited.
 *
 *  @brief      Defines the CMessageLogStreamBuf class.
 * 
 *  @licence    This software is provided 'as-is', without any express 
 *              or implied warranty.
 *
 *              In no event will the author be held liable for any damages 
 *              arising from the use of this software.
 *  
 *              Permission is granted to anyone to use this software 
 *              for any purpose, including commercial applications, 
 *              and to alter it and redistribute it freely, subject to 
 *              the following restrictions: 
 *
 *              - The origin of this software must not be misrepresented; 
 *                you must not claim that you wrote the original software. 
 *                If you use this software in a product, an acknowledgment 
 *                in the product documentation is requested but not required. 
 *
 *              - Altered source versions must be plainly marked as such, 
 *                and must not be misrepresented as being the original software. 
 *                Altered source is encouraged to be submitted back to 
 *                the original author so it can be shared with the community. 
 *                Please share your changes. 
 *
 *              - This notice may not be removed or altered from any 
 *                source distribution. 
 */

#ifndef LOG_STREAMBF_HPP
#define LOG_STREAMBF_HPP

#include <streambuf>

namespace debug
{
namespace log
{

class CMessageLog ;

/* --- CLASS DEFINITION: CMessageLogStreamBuf ------------------------- */

/// For internal use only.
/** @class CMessageLogStreamBuf log/streambf.hpp "log/streambf.hpp"
 *  This streambuf class simply acts as a proxy and passes received characters 
 *  on to its parent CMessageLog object for processing.
 */

class CMessageLogStreamBuf 
    : public std::streambuf 
{

private:                    // --- PRIVATE INTERFACE ---

// constructors:
                            CMessageLogStreamBuf( CMessageLog& msgLog ) ;
// destructor:
    virtual                 ~CMessageLogStreamBuf() ;

// streambuf overrides:
    virtual int             overflow( int ch ) ;
    virtual int             underflow() ;

// copy/assignment (undefined):
                            CMessageLogStreamBuf( const CMessageLogStreamBuf& rhs ) ;
    CMessageLogStreamBuf&   operator=( const CMessageLogStreamBuf& rhs ) ;

// data members:
private:
    CMessageLog&            mMsgLog ; ///< Parent message log stream.

// friends:
friend class CMessageLog ;

} ;

/* -------------------------------------------------------------------- */
} // End namespace log
} // End namespace debug

#endif // LOG_STREAMBF_HPP
