#ifndef _BRUSH_H
#define _BRUSH_H

#include <Tools.h>

namespace cache
{
	/*
	*	This stores the parameters of the current brush it is a singleton
	*/
	class Brush
	{

	public:
		enum Type{Any,All};
		/**
		* Constructs a new brush.
		* \return 
		*/
		Brush();

		/**
		* Constructs a new brush
		* \param e1 
		* \param e2 
		* \param lod 
		* \param type 
		* \return 
		*/

		explicit Brush(double e1,double e2,double lod,int numberOfDimensions,int type);

		/**
		* Constructs a new brush
		* \param e1 
		* \param e2 
		* \param lod 
		* \param brushType 
		* \return 
		*/
		explicit 
			Brush(double e1,double e2,double lod,unsigned int numberOfDimensions,Type brushType);

		/**
		* Define the copy constructor
		* \param &brush 
		* \return 
		*/
		Brush(Brush &brush);

		Brush(int numberOfDimensions);

		Brush& operator =(Brush &brush);

		bool operator==(Brush &b);

		double getLeftExtent();
		void setLeftExtent(double e1)
		{
			this->e1 = e1;
		}

		double getRightExtent();
		void setRightExtent(double e2)
		{
			this->e2 = e2;
		}

		double getLevelOfDetail();

		Type getBrushType();	

		//Checks to see if the value is really in the brush
		bool inBrush(Tools::Geometry::IShape& iShape);

		Tools::Geometry::IShape& getShape();

		/**
		* Returns the condition used to get the brush
		* \return 
		*/
		std::string getPredicate();

		/**
		* Sets the level of the detail for the brush
		* \param lod 
		*/
		void setLevelOfDetail(double lod);


		/**
		* Deletes the iShape object
		* \return 
		*/
		~Brush();

	private:
		double e1;
		double e2;
		double lod;
		Type brushType;	
		Tools::Geometry::IShape *iShape;	
		int numberOfDimensions;
	};
}
#endif //BRUSH_H