#ifndef _BRUSH_STORE_H
#define _BRUSH_STORE_H

#include <deque>
namespace cache
{

	//Forward References
	class Brush;

	/*
	* This is a central repository that stores the history of the user
	* brushes and can be used for prediction purposes both by the
	* prefetcher and the replacement policy
	*/
	class BrushStore
	{
	public:
		/**
		* Stores the number of brushes to be stored in 
		* the brush store
		* \param size default size is 2
		* \return 
		*/
		BrushStore(int size=2);

		/**
		* Copies the brush by value into the brush store
		* \param *brush 
		* \return 
		*/
		void addBrush(Brush *brush);

		/**
		* Gets the ith brush from the brush store
		* \param index 
		* \return 
		*/
		Brush* getBrush(unsigned int index);

		/**
		* Destroys the BrushStore
		* \return 
		*/
		~BrushStore();

	private:
		//Store the maxSize of the store.
		unsigned int maxSize;	
		//Store the queue
		std::deque<Brush*>* dequeBrush;
	};
}
#endif