#ifndef _CACHE_BUILDER_H
#define _CACHE_BUILDER_H

#include <cache/CacheManager.h>
#include <DataBaseLibrary/Connection.h>
#include <iostream>
#include <cache/LRUReplacement.h>
#include <cache/ReplacementPolicy.h>
#include <cache/CacheRegion.h>
#include <sstream>
#include <cache/BrushStore.h>
#include <fstream>
#include <cache/ConfigurationStore.h>
#include <cache/CacheEntry.h>
#include <log/log.h>
#include <cache/DistanceReplacement.h>


namespace cache
{

	//Stores Forward References
	class Loader;	
	class PrefetchController;	
	class Prefetcher;	
	class CacheController;	
	class ICacheController;
	class CacheManager;	
	class CacheConfigStore;
	class DBConfigStore;
	class BrushStore;	
	class StatisticsGatherer;	
	class LoadRequester;;	
	class Cache;			
	class AbstractQueryFactory;
	class IIndex;	
	class Schema;

	/*
	*	A Factory class that 
	*  builds the cache connects the different componets together
	*  It can build the cache for the following 3 configurations
	* 1) No Prefetcher
	* 2) Fixed Prefetcher
	* 3) Adaptive Prefetcher
	*/
	/*
	/*The Data base connection parameters
	* username
	* passwd
	* dsn source
	*/
	/*	The input parameters the class requires for Cache
	* 1) Cache Size 
	* 2) fillFactor = 0.7; //Default Value = 0.7
	* 3) indexCapacity = 2000; //Need to have some default value
	* 4) leafCapacity =  2000; //Need to have some default value
	* 5) Replacement policy to use 
	* ============================================================
	* Prefetcher
	* 1) Prefetcher Enabled or not
	* 2) Prefetch adaptive or fixed
	* 3) Initial Prefetch strategy to be used. //Default value should be there
	* 3) brush store size //Default value is 2
	* 
	*/
	/*

	*/
	class CacheBuilder
	{
		template <class Policy,class Value>
		class Builder
		{					
		friend class CacheBuilder;

		public:
		
			Builder(CacheConfigStore* cacheConfigStore,
					DBConfigStore* dbConfigStore);

			ICacheEntry* makeBuffer()
			{
				CacheEntry<Value>* buffer = 
					new CacheEntry<Value>[cacheConfigStore->getCacheSize()];

				for(unsigned int i=0;i<cacheConfigStore->getCacheSize();i++)
					buffer[i].setID(i);

				return(buffer);
			}
			/*
			 *	Retunrs the instantaited prefetch controller.
			 */
			PrefetchController* getPrefetchController()
			{
				return(this->prefetchController);
			}
			
			ReplacementPolicy* makeReplacementPolicy(ICacheEntry* buffer)
			{
				//Type cast
				CacheEntry<Value>* buf = (CacheEntry<Value>*) buffer;

				if(this->cacheConfigStore->replacementPolicy == "LRU")
				{
				//Makes a new replacement policy
				this->replacementPolicy = new Policy(this->cache,
													 buf,
													 cacheConfigStore->getCacheSize(),
													 this->index);
				}
				if(this->cacheConfigStore->getReplacementPolicy() == "Distance")
				{
				this->replacementPolicy = new Policy(this->cache,
													 buf,
													 cacheConfigStore->getCacheSize(),
													 this->index);
				}
				return(this->replacementPolicy);	
			}

			void makeCacheManager();
			/**
			* Makes the new Prefetcher
			*/
			void makePrefetcher();		

			
			~Builder();
			//Store the reference to the constructed loader
			Loader* loader;
			//Stores the reference to the prefetch controller
			PrefetchController* prefetchController;
			//Stores the referene to the prefetcher
			Prefetcher* prefetcher;
			//Stores the reference to the CacheController
			CacheController* cacheController;
			//Stores the reference to the CacheManager
			CacheManager *cacheManager;	
			//Store the reference to the Configuration Store
			CacheConfigStore* cacheConfigStore;
			//Stores the reference to the db configuration store
			DBConfigStore* dbConfigStore;
			//Stores the referene to the brush store
			BrushStore *brushStore;
			//Stores the reference to the Statistics Gatherer
			StatisticsGatherer * statisticsGatherer;
			//Stores the reference to the LoadRequester
			LoadRequester *loadRequester;
			//Stores the reference to the constructed cache
			Cache* cache;
			//Stores the reference to the replacement Policy
			ReplacementPolicy* replacementPolicy;
			//Stores the reference to the connection
			::Connection* connection;
			//Stores the reference to the statement
			::Statement* statement;
			//Stores the reference to the query factory
			AbstractQueryFactory* queryFactory;
			//Stores the reference to the index structure used in the cache
			IIndex* index;
			//Stores the reference to the performance statistics object
			PerformanceStatistics *perStatistics;
			//Stores the reference to the schema made by the db config store.
			Schema* schema;
		};

	public:
		CacheBuilder(CacheConfigStore* cacheConfigStore,
					DBConfigStore* dbConfigStore);		
		CacheBuilder();

		
		ICacheController* getCacheController();
		
		PrefetchController* getPrefetchController();
		/*
		 *	Deletes the cache builder
		 */
		~CacheBuilder();
		

	private:
		//Have to add one for all the replaement policies
		Builder<LRUReplacement,LRUReplacementValue>* bLRU;		

		Builder<DistanceReplacement,bool>* bDistance;

	};
}
#endif //_CACHE_BUILDER_H
