#ifndef _CONFIGURATION_STORE_H
#define _CONFIGURATION_STORE_H
#include <string>
#include <cache/Schema.h>
namespace cache
{
	//Forward References
	class ConfigFileParser;
	class DBFileParser;


	class DBConfigStore
	{	
	public:
		/*
		* Constructs the DBConfigStore
		*/
		DBConfigStore(){			
			this->userName = "xmdv";
			this->passwd = "xmdv";
			this->dsnsource = "odbc_oravis";
		}

		/*
		* This returns the schema.
		* Note the one who retrieves the schema has to take care of
		* deleteing it because this is a dynamically allocated object.
		*@return schema
		*/
		Schema* getSchema()
		{
			return(this->schema);
		}

		//Stores the connection parameters to the data base.
		//Stores the user name
		std::string userName;
		//stores the passwd
		std::string passwd;
		//Stores the dsn source
		std::string dsnsource;
		//Stores the table name
		std::string tableName;
		//Stores the table name of the contour
		std::string contourTable;
		//Stores the pointer to the schema object
		Schema *schema;
		

		~DBConfigStore()
		{
			delete(this->schema);
		}
	
	
	};

	/*
	* It stores all the cache configuration parameters
	*/
	class CacheConfigStore
	{	
	public:
		//Setting the default values of the parameters. Note the size of the 
		//cache has to be set explicity.
		CacheConfigStore(unsigned long sizeOfCache){

			this->cacheSize = sizeOfCache;
			//Default Value of fragmentation Thresold
			this->fragmentationThreshold = 12;

			this->fillFactor = 0.7;
			this->leafCapacity = 10;
			this->indexCapacity = 10;
			this->queryType = "Simple";

			this->replacementPolicy = "LRU";
			this->prefetchStrategy = Noprefetch;
			this->prefetchType = Fixed;
			this->sizeOfBrushStore = 2;

			this->memoryIndex = "Sequential";	
			//These don't matter but just in case 
			this->fillFactor = 0.7;
			this->leafCapacity = 10;
			this->indexCapacity = 10;
			////////////////////////////////////
			this->queryType = "Simple";
		}

		CacheConfigStore()
		{
			this->fragmentationThreshold = 5;
		}

		//Describes the type of Prefetcher to use
		enum PrefetcherType {Fixed=0,Adaptive=1};
		//Describes the initial Prefetch Strategy
		enum PrefetchStrategy {Noprefetch=0,Random =1,Direction=2,Focus=3};

		unsigned long getCacheSize()
		{
			return(this->cacheSize);
		}
		double getRTreeFillFactor()
		{
			return(this->fillFactor);
		}

		unsigned int getRTreeIndexCapacity()
		{
			return(this->indexCapacity);
		}

		unsigned int getRTreeLeafCapacity()
		{
			return(this->leafCapacity);
		}

		std::string getReplacementPolicy()
		{
			return(this->replacementPolicy);
		}

		PrefetcherType getPrefetcherType()
		{
			return(this->prefetchType);
		}

		PrefetchStrategy getPrefetchStrategy()
		{
			return(this->prefetchStrategy);
		}

		unsigned int getSizeOfBrushStore()
		{
			return(this->sizeOfBrushStore);
		}


		~CacheConfigStore()
		{}

		//Cache Parameters
		unsigned int cacheSize; 
		double fillFactor;
		unsigned int indexCapacity; //Need to have some default value
		unsigned int leafCapacity; //Need to have some default value
		std::string replacementPolicy;	
		unsigned int fragmentationThreshold;

		//Prefetch parameters
		PrefetcherType prefetchType; //adpative
		PrefetchStrategy prefetchStrategy;
		unsigned int sizeOfBrushStore;		
		//Stores the type of the query
		std::string queryType;
		//Stores the memory index type
		std::string memoryIndex;
		
	};

}
#endif //_CONFIGURATION_STORE_H
