#ifndef _QUERY_FACTORY_H
#define _QUERY_FACTORY_H

#include<stdio.h>
#include<string>
#include<sstream>
#include <vector>
#include <Schema.h>
#include <iostream>
#include <fstream>

namespace cache
{

	//Forward References
	class RemainderRegion;
	class Brush;



	class AbstractQueryFactory
	{
	
	protected:
		AbstractQueryFactory(Schema *schema,std::string tableName);
	
		std::string tableName;

		//Stores the schema that the query factory uses to form the query
		Schema *schema;
		//Stores the select caluse used by the subclasses
		std::string selectClause;
		//Stores the reference to the output file where all the queries are sent
		std::ofstream outputFile;

	public:

		std::string makeSelectClause();

		/**
		* Makes the query
		* \param &vecRemRegions 
		* \param tableName 
		* \return 
		*/
		virtual std::string makeQuery(std::vector<RemainderRegion*>& vecRemRegions) = 0;

		/**
		* Makes the query from the brush
		* \param brush 
		* \return 
		*/
		virtual std::string makeQuery(Brush& brush) = 0;


		/**
		* Destroys the AbstractQueryFactory.
		* \return 
		*/
		virtual ~AbstractQueryFactory(){}



	};

	class QuerySimpleFactory : public AbstractQueryFactory
	{
	private:
		QuerySimpleFactory(Schema *schema,std::string tableName);

		static QuerySimpleFactory *qsf;

		//Stores the schema that the query factory uses to form the query
		Schema *schema;

	public:


		/**
		* This is used to make the query factory
		* \param schema 
		* \param tableName 
		* \return 
		*/
		static QuerySimpleFactory* makeQueryFactory(Schema* schema,
			std::string tableName);


		/**
		* Makes the query
		* \param &vecRemRegions 
		* \param tableName 
		* \return 
		*/
		std::string makeQuery(std::vector<RemainderRegion*>& vecRemRegions);

		/**
		* Makes the query from the brush
		* \param brush 
		* \return 
		*/
		std::string makeQuery(Brush& brush);

		/**
		* Destorys the query factory
		* \return 
		*/
		~QuerySimpleFactory();
		
		/**
		 * Returns the reference to the pointer
		 * \return 
		 */
		static QuerySimpleFactory*& getReference();


	};

	/*
	*	QueryFactory is a singleton
	* It is used to make the query to be sent to the 
	* data base, It uses the RemainderRegions and the 
	* table Name to form the query. Note this class
	* can be used only for the structure based brush.
	*/
	class QueryGeomFactory : public AbstractQueryFactory
	{

	private:
		QueryGeomFactory(Schema *schema,std::string tableName);

		static QueryGeomFactory *qf;	
	public:
		
		/**
		* 
		* \returns a reference to a pointer
		*/
		static QueryGeomFactory*& getReference();

		/**
		* This is used to make the query factory
		* \param schema 
		* \param tableName 
		* \return 
		*/
		static QueryGeomFactory* makeQueryFactory(Schema* schema,
			std::string tableName);


		/**
		* Makes the query
		* \param &vecRemRegions 
		* \param tableName 
		* \return 
		*/
		std::string makeQuery(std::vector<RemainderRegion*>& vecRemRegions);

		/**
		* Makes the query from the brush
		* \param brush 
		* \return 
		*/
		std::string makeQuery(Brush& brush);

		/**
		* Destorys the query factory
		* \return 
		*/
		~QueryGeomFactory();

		/**
		* Returns the gemoetry predicate given an 
		* Remainder region
		* \return 
		*/
		std::string getGeometryPredicate(double leftExtent,
			double rightExtent,
			double levelOfDetail);

	
	};

	class QueryModifier
	{
		public:
			unsigned int type; //true if asc and 0 if desc

			QueryModifier(const unsigned int type)
			{
				this->type = type;
			}
		
		std::string modifyQuery(std::string query)
		{
			std::string result = query;
			switch(type)
			{
			
				case 0:					
					break;
				case 1:
					result +=" order by X asc";
					break;
				case 2:
					result +=" order by X desc";
					break;		
			}			
		 return(result);
		}
	};
}
#endif //_QUERY_FACTORY_H