#ifndef _REGION_FACTORY_H
#define _REGION_FACTORY_H

#include <CacheEntry.h>
#include <RemainderRegion.h>
#include <Brush.h>
#include <vector>
#include <list>
#include <CacheRegion.h>
#include <ICacheEntry.h>

namespace cache
{

	class AbstractRegionFactory
	{	
	protected:
		enum Place {left=0,right=1,both=2};	

	public :	
		/**
		* Note it is the responsibility of the caller to delete all the 
		* remainder regions and also the vector from the return list.
		* \param list 
		* \param brush 
		* \return 
		*/
		virtual std::vector<RemainderRegion*>& makeRegions(
			std::list<ICacheEntry*>& listCacheObjects,
			Brush& brush);

		virtual ~AbstractRegionFactory();

	protected:
		virtual void makeInBetweenRegions(
			std::vector<RemainderRegion*> &vecRemainderRegion,
			std::vector<CacheRegion>& vecCacheRegion,
			double levelOfDetail);

		void makeCacheRegions(std::vector<CacheRegion>& vecCacheRegion,		
			std::list<ICacheEntry*>& listCacheObjects);

		RemainderRegion* make(double leftExtent,
			double rightExtent,
			double levelOfDetail,
			Brush::Type type,
			Place place);
		void removeRegions(std::vector<RemainderRegion*> *vecRemainderRegion);

	protected:
		std::vector<RemainderRegion*>* vecRemainderRegionPtr;		
	};

	/*
	*	A singleton class makes the remainder regions
	* Uses the class cache Region and the frinend function defined in
	* CacheRegion.h
	*/

	class AscRegionFactory : public AbstractRegionFactory
	{

	public :		
		AscRegionFactory();
		/**
		*
		* \return RegionFactory<T>*
		*/
		static AscRegionFactory* getInstance();

		/**
		*
		* \return RegionFactory<T>*
		*/
		static AscRegionFactory*& getReference();

	private:			
		static AscRegionFactory* regionFactoryPtr;	
	};



	/**Note this might not be the most efficient implementation but
	* But this is not my focus right now.
	*/

	class DescRegionFactory : public AbstractRegionFactory
	{

	public :		
		DescRegionFactory();
		/**	
		* \param list 
		* \param brush 
		* \return 
		*/
		virtual std::vector<RemainderRegion*>& makeRegions(
			std::list<ICacheEntry*>& listCacheObjects,
			Brush& brush);
		/**
		*
		* \return RegionFactory<T>*
		*/
		static DescRegionFactory* getInstance();
		
		/**
		*
		* \return RegionFactory<T>*
		*/
		static DescRegionFactory*& getReference();

	private:			
		static DescRegionFactory* regionFactoryPtr;	
	};

}
#endif //_REGION_FACTORY_H