#ifndef _SCHEMA_H
#define _SCHEMA_H
#include <string>
#include <vector>
#include <sstream>
#include <cache/Record.h>
#include <DataBaseLibrary/ResultSet.h>

namespace cache
{

	/*
	*	This class stores the schema information 
	*  Also it decides on the order in which the 
	*  columns will be retreived from the data base.
	*  This information is used to generate the query for the data base
	*  To convert the result set to a record.
	*  This is the central place where the order in which the attributes are retreived 
	*  is decided.
	* Also note that all the meta columns are renamed so even if the column names
	* are different it is still going to work.
	*/
	class Column
	{
	public:	
		//The first
		enum Type{meta,data};
		/*
		* Copy constructor
		*/
		Column(Column& column)
		{
			this->columnName = column.columnName;
			this->t = t;
		}

		Column& operator =(Column &column)
		{
			this->columnName = column.columnName;
			this->t = t;
			return(*this);
		}

		Column(Type t,std::string name):columnName(name),t(t)
		{}
		
		Column(Column::Type t):t(t)
		{}
		Column(){}
		//Stores the name of the columns
		std::string columnName;	
		//Stores the type
		Type t;
	};

	class Schema
	{	
	public:
		/*
		* Makes the schema
		* Copy constructor
		*/
		Schema(Schema &schema)
		{
			this->columnX1 = schema.columnX1;
			this->columnX2 = schema.columnX2;
			this->columnY1 = schema.columnY1;
			this->columnY2 = schema.columnY2;
			this->columnN = schema.columnN;
			this->numberOfDimensions = schema.numberOfDimensions;

			this->selectString = schema.selectString;			
			for(unsigned int i = 0;i<schema.dataColumns.size();i++)
				this->dataColumns.push_back(new Column(*schema.dataColumns[i]));
		}

		/*
		* Makes the schema
		*/
		Schema()
		{
			this->columnX1.t = Column::meta;
			this->columnX2.t = Column::meta;
			this->columnY1.t = Column::meta;
			this->columnY2.t = Column::meta;	
			this->columnN.t = Column::meta;				

			/*
			this->columnX1.t = Column::Type::meta;
			this->columnX2.t = Column::Type::meta;
			this->columnY1.t = Column::Type::meta;
			this->columnY2.t = Column::Type::meta;	
			this->columnN.t = Column::Type::meta;				
			*/
			this->numberOfDimensions = 0;
		}

		/*
		*	Returns the select string
		*/
		std::string getSelectString()
		{
			return(this->selectString);
		}


		/*
		*	Makes the record from the data base
		*/
		Record* makeRecord(ResultSet* resultSet)
		{
			Record* record = new Record((unsigned int)this->dataColumns.size());		

			//Fetch the rectangle 
			record->e1 = resultSet->getDouble(0); //x1
			record->e2 = resultSet->getDouble(1); //x2
			record->radius = resultSet->getDouble(2); //y1
			record->parent_radius = resultSet->getDouble(3);//y2
			//Note I am using double even for getting an integer, because the
			//Wrapper does not handle integers very well.
			record->N = (unsigned int) resultSet->getInt(4);
		
			for(unsigned int i=0;i<this->numberOfDimensions;i++)
				record->D[i] =  resultSet->getDouble(i+5);

			return(record);
		}
		/*
		*	Deleting all the columns in the schema
		*/
		virtual ~Schema()
		{
			for(unsigned int i=0;i<dataColumns.size();i++)
			{
				delete(dataColumns[i]);
			}
		}

		/*
		*	Makes the select string
		*/
		void makeSelectString()	
		{
			std::ostringstream selectStream;
			selectStream<<this->columnX1.columnName<<" "<<"x"<<","
				<<this->columnX2.columnName<<" "<<"y"<<","
				<<this->columnY1.columnName<<" "<<"l"<<","
				<<this->columnY2.columnName<<" "<<"pl"<<","
				<<this->columnN.columnName;


			std::vector<Column*>::iterator i;

			for(i=dataColumns.begin();i!=dataColumns.end();i++)
			{
				selectStream<<","<<(*i)->columnName;
			}
			this->selectString = selectStream.str();
		}

		//Stores the data columns
		std::vector<Column*> dataColumns;
		//Stores the generated select strings
		std::string selectString;
		//Stores the meta columns	
		Column columnX1;
		//Stores the columnY
		Column columnX2;
		//Stores the  radius column
		Column columnY1;
		//Stores the  radius of the parent column
		Column columnY2;
		//Stores the number of leave nodes that encompass the node.
		Column columnN;
		//Stores the name of the geometry column
		Column columnShape;
		//Stores the number of dimensions
		unsigned int numberOfDimensions;
	};
}
#endif
