// MetaData.h: interface for the MetaData class.
//
//////////////////////////////////////////////////////////////////////

#ifndef __METADATA_H__
#define __METADATA_H__

#include <string>
#include <iosfwd>
#include <vector>

struct IDimension;
typedef std::vector<IDimension*> DimVec;
typedef DimVec::iterator		 DimVecIter;
typedef DimVec::const_iterator	 DimVecCIter;

class MetaData : public IMeta
{
public:
	MetaData();
	virtual ~MetaData();

public:
	bool unmarshal(std::istream &is, ISerializable *parent = 0);
	bool marshal(std::ostream& s, ISerializable *parent = 0);

public:
	int getDimensionCount() const;
	void setDimensionCount(int dimensions);

	int getDataPointCount() const;
	void setDataPointCount(int datapoints);

	IDimension *getDimension(int dim) const;
	IDimension *getDimension(const char* dimName) const;
	
	IDimension*	createDimension(const char* dimName,Type dimType) const;

	void addDimension(IDimension *dimension);
	void		remDimension(IDimension *dimension) {}
	
private:
	class MetaDataHandler;
	friend class MetaDataHandler;
	MetaDataHandler *mHandler;

	// metadata content
	int		mDimensionsCount;
	int		mDataPointsCount;
	DimVec mDimensions;
};

#endif //__METADATA_H__
