// Numeric.h: interface for the Numeric class.
//
//////////////////////////////////////////////////////////////////////

#ifndef __NUMERIC_H__
#define __NUMERIC_H__

#include <meta/BasicTypes.h>

class Numeric : public BasicTypeValue
{
public:
	static Type getTypeId() {
		if ( mNumTypeId.isBad() ) {
			mNumTypeId = Type::registerAbstractType(
				BasicTypeValue::getTypeId(),"numeric");
		}
		return mNumTypeId;
	}
protected:
	//	Number() {}
private:
	static Type mNumTypeId;
};

class Integer : public Numeric
{
public:
	Integer(const char *str) 
	{ mValue = atoi(str); }
	Integer(int i = 0)
	{	mValue = i;	}
	Integer(const Integer& iv)
	{	mValue = iv.mValue;	}
	
	static Type getTypeId() {
		if ( mIntTypeId.isBad() ) {
			mIntTypeId = Type::registerType(
				Numeric::getTypeId(),"integer",
				createIntegerValue);
		}
		return mIntTypeId;
	}
	
	static IValue* createIntegerValue(const char* str, void*) { return new Integer(str); }
	
	Type getType() const { return Integer::getTypeId(); }
	IValue* clone() const { return new Integer(*this); }

	// --- String representation support
	// get the string representation of the value
	// this is usually shorter representation of the value
	// (for eg. string -> shorter version of the complete string, uri)
	const char* getStringRepresentation() const
	{
		std::stringstream ss; ss << mValue;
		return ss.str().c_str();
	}
	
	// get the actual contents of the value
	//  eg. 1. uri -> will give the contents of the file.
	//      2. string -> will give the complete string
	// NOTE the difference from getStringRepresentation()
	const char* getStringContents() const
	{
		std::stringstream ss; ss << mValue;
		return ss.str().c_str();
	}
	
protected :
	int mValue;
	static Type mIntTypeId;
	
};

class Real : public Numeric
{
public:
	Real(const char *str) 
	{ mValue = static_cast<float>(atof(str)); }
	Real(float i = 0)
	{	mValue = i;	}
	Real(const Real& iv)
	{	mValue = iv.mValue;	}
	
	static Type getTypeId() {
		if ( _classTypeId.isBad() ) {
			_classTypeId = Type::registerType(
				Numeric::getTypeId(),"real",
				createRealValue);
		}
		return _classTypeId;
	}
	
	static IValue* createRealValue(const char* str, void*) { return new Real(str); }
	
	Type getType() const { return Real::getTypeId(); }
	IValue* clone() const { return new Real(*this); }
	// --- String representation support
	// get the string representation of the value
	// this is usually shorter representation of the value
	// (for eg. string -> shorter version of the complete string, uri)
	const char* getStringRepresentation() const
	{
		std::stringstream ss; ss << mValue;
		return ss.str().c_str();
	}
	
	// get the actual contents of the value
	//  eg. 1. uri -> will give the contents of the file.
	//      2. string -> will give the complete string
	// NOTE the difference from getStringRepresentation()
	const char* getStringContents() const
	{
		std::stringstream ss; ss << mValue;
		return ss.str().c_str();
	}
	
protected :
	float mValue;
	static Type _classTypeId;
	
};

#endif //__NUMERIC_H__
