#ifndef _PRINTER_H
#define _PRINTER_H

#include <string>
#pragma warning(disable: 4786 4250 4503 4244)

#include <DOM/SAX2DOM/SAX2DOM.h>
#include <SAX/helpers/CatchErrorHandler.h>
#include <DOM/Utils/Stream.h>
#include <vector>
#include <iostream>
#include <map>

namespace cache
{
	//Forward References
	class DBConfigStore;
	class CacheConfigStore;
}

//Forward References
class Printer;
/*
 *	This parser also acts as a factory and make the Printers
 */
class PrintFileParser
{
public:	
	PrintFileParser(std::string fileName,
					cache::CacheConfigStore* configStore,
					cache::DBConfigStore* dbConfigStore);

	/**
	* Makes the Printers;
	*/
	void makePrinters(DOM::Document<std::string>& doc);

	std::map<std::string,Printer*>* getMap();

private:	
	std::map<std::string,Printer*>* printerMap;
	cache::DBConfigStore* dbConfigStore;
	cache::CacheConfigStore* cacheConfigStore;
};


class Printer
{
public:
	Printer(cache::DBConfigStore* dbConfigStore,
			cache::CacheConfigStore *cacheConfigStore,
			DOM::NodeList<std::string>& nodeList);

	Printer::Printer(cache::DBConfigStore* dbConfigStore,
		cache::CacheConfigStore *cacheConfigStore,
		DOM::Node<std::string>& node);

	//Prints the output to the output stream
	friend std::ostream& operator <<(std::ostream& outputStream,Printer& printer);

	std::string getID();

	void printName(std::ostream &outputStream);	

	void virtual printValue(std::ostream &outputStream)=0;

	virtual void setValue(std::string value);

	virtual void setValue(double value);

	virtual void setValue(unsigned long value);

	virtual void setValue(unsigned int value);

	virtual void setValue(int value);

	virtual ~Printer(){};
	
protected:
	bool isEnabled;
	std::string id;
};


class CacheSizePrinter : public Printer
{
public:
	CacheSizePrinter(cache::DBConfigStore* dbConfigStore,
					cache::CacheConfigStore *cacheConfigStore,
					DOM::NodeList<std::string>& nodeList);

	void printValue(std::ostream &outputStream);		

private:
	unsigned long value;
	
};

class InputUserTracePrinter : public Printer
{
public:
	InputUserTracePrinter (cache::DBConfigStore* dbConfigStore,
		cache::CacheConfigStore *cacheConfigStore,
		DOM::NodeList<std::string>& nodeList);

	void printValue(std::ostream &outputStream);		
	
	void setValue(std::string inputUserTrace);

private:
	std::string value;

};

class ReplacementPolicyPrinter: public Printer
{
public:
	ReplacementPolicyPrinter(cache::DBConfigStore* dbConfigStore,
		cache::CacheConfigStore *cacheConfigStore,
		DOM::NodeList<std::string>& nodeList);

	void printValue(std::ostream &outputStream);

	std::string getID();
private:
	std::string value;
};

class SecondaryIndexPrinter: public Printer
{
public:
	SecondaryIndexPrinter(cache::DBConfigStore* dbConfigStore,
		cache::CacheConfigStore *cacheConfigStore,
		DOM::NodeList<std::string>& nodeList);

	void printValue(std::ostream &outputStream);
	
private:
	std::string value;
};

class MainMemoryIndexPrinter: public Printer
{
public:
	MainMemoryIndexPrinter(cache::DBConfigStore* dbConfigStore,
		cache::CacheConfigStore *cacheConfigStore,
		DOM::NodeList<std::string>& nodeList);

	void printValue(std::ostream &outputStream);

	~MainMemoryIndexPrinter();
private:
	std::string value;

	std::vector<Printer*>* children;

};


class RTreeNodeSizePrinter: public Printer
{
	public:
	RTreeNodeSizePrinter(cache::DBConfigStore* dbConfigStore,
		cache::CacheConfigStore *cacheConfigStore,
		DOM::Node<std::string>& node);

	void printValue(std::ostream &outputStream);

private:
	unsigned int value;	
};
class RTreeFillFactorPrinter: public Printer
{
	public:
	RTreeFillFactorPrinter(cache::DBConfigStore* dbConfigStore,
							cache::CacheConfigStore *cacheConfigStore,
							DOM::Node<std::string>& node);

	void printValue(std::ostream &outputStream);

private:
	double value;
};

class RTreeIndexPrinter: public Printer
{
	public:
	RTreeIndexPrinter(cache::DBConfigStore* dbConfigStore,
					cache::CacheConfigStore *cacheConfigStore,
					DOM::Node<std::string>& node);

	void printValue(std::ostream &outputStream);

private:
	unsigned int value;	
};


class PrefetchPrinter : public Printer
{
public:
	PrefetchPrinter(cache::DBConfigStore* dbConfigStore,
		cache::CacheConfigStore *cacheConfigStore,
		DOM::NodeList<std::string>& nodeList);

	void printValue(std::ostream &outputStream);		

private:
	std::string value;
};

//Output Printers
class TotalTuplesPrinter : public Printer
{
public:
	TotalTuplesPrinter(cache::DBConfigStore* dbConfigStore,
		cache::CacheConfigStore *cacheConfigStore,
		DOM::NodeList<std::string>& nodeList);

	void printValue(std::ostream &outputStream);		

	void setValue(unsigned long totalTuples);

private:
	unsigned long value;

};

class TotalLatencyPrinter : public Printer
{
public:
	TotalLatencyPrinter(cache::DBConfigStore* dbConfigStore,
		cache::CacheConfigStore *cacheConfigStore,
		DOM::NodeList<std::string>& nodeList);

	void printValue(std::ostream &outputStream);

	void setValue(unsigned long totalLatency);

private:
	unsigned long value;
};

class TotalSearchLatencyPrinter: public Printer
{
public:
	TotalSearchLatencyPrinter(cache::DBConfigStore* dbConfigStore,
		cache::CacheConfigStore *cacheConfigStore,
		DOM::NodeList<std::string>& nodeList);

	void printValue(std::ostream &outputStream);	
		
	void setValue(unsigned long totalSearchLatency);	

private:
	unsigned long value;
};


class AvgLatencyPrinter: public Printer
{
public:
	AvgLatencyPrinter(cache::DBConfigStore* dbConfigStore,
					cache::CacheConfigStore *cacheConfigStore,
					DOM::NodeList<std::string>& nodeList);

	void printValue(std::ostream &outputStream);

	void setValue(double avgLatency);	

private:
	double value;

};

class MedianLatencyPrinter : public Printer
{
public:
	MedianLatencyPrinter(cache::DBConfigStore* dbConfigStore,
		cache::CacheConfigStore *cacheConfigStore,
		DOM::NodeList<std::string>& nodeList);

	void printValue(std::ostream &outputStream);

	void setValue(double medianLatency);	

	private:
		double value;

};
class HitRatioPrinter : public Printer
{
public:
	HitRatioPrinter	(cache::DBConfigStore* dbConfigStore,
					cache::CacheConfigStore *cacheConfigStore,
					DOM::NodeList<std::string>& nodeList);

	void printValue(std::ostream &outputstream);

	void setValue(double hitRatio);

private:
	double value;

};

class FragThresholdPrinter : public Printer
{
	public:
		FragThresholdPrinter(cache::DBConfigStore* dbConfigStore,
							cache::CacheConfigStore *cacheConfigStore,
							DOM::NodeList<std::string>& nodeList);

		void printValue(std::ostream &outputstream);

		void setValue(unsigned int fragThreshold);

private:
	unsigned int value;
};

#endif //_PRINTER_H