///////////////////////////////////////////////////////////////////////////////
/// \file CFlatPixel.h
/// \brief Draw data using a pixel oriented visualization
/// \date 5/6/2004
/// \author Anilkumar Patro
///////////////////////////////////////////////////////////////////////////////

#ifndef _CFLATPIXEL_H
#define _CFLATPIXEL_H

// SYSTEM INCLUDES
//
#include <vector>

// PROJECT INCLUDES
//
#include <xmdv/PixelOriented.h>
#include <xmdv/CFlatDisplay.h>

// FORWARD DECLARATIONS
//
class colormap;
class Image;
class Subwindow;
class Segment;

///
/// Flat Query-independent Pixel Oriented Visualization implementation
///
/// \date 03-26-2004
/// \author anilkpatro
///
class CFlatPixel : public CFlatDisplay, public CPixelOriented
{
public:
/*
	enum DisplayType {
		PDT_SPIRAL,
		PDT_SEGMENTS,
	};

	enum PlacementType {
		PPT_NORMAL,
		PPT_MDS,
	};
*/
// LIFECYCLE
//
	CFlatPixel(CFlatDisplayInformation *flat_info);
	CFlatPixel(const CFlatPixel& c);
	virtual ~CFlatPixel();

// OPERATIONS
//
	// ---- CFlatDisplay implementation ---
	void		Draw();

	/// 
	/// Draw each dimension (subwindow) of pixel oriented display
	/// 
	void		DrawAxes();

	void		DrawOthers();

	void		FreeSpace();

	//event handlers
	void		InputMouseMotion(int x, int y);
	void		InputButtonPress(int button, int x, int y);
	void		InputButtonRelease(int button, int x, int y);

	// ---- CPixelOriented implementation ----
   	void		ComputeQueryDist(int sortedDim=0,bool isAscending=true);

	void		ShowColormap(bool bShow);
	void		ShowDerived(bool bShow);

// ACCESS
//
	void		SetDisplay(DisplayType dt);
	DisplayType	GetDisplay(void) const;

	int			GetSortedDim() const;
	void		SetSortedDim(int dim);

	int			GetSelectedDim() const;
	void		SelectDim(int dim);

	float		GetDimDistortion(int dim) const;
	void		SetDimDistortion(int dim, float dist);

	bool		GetSortDimOrder() const;
	void		SetSortDimOrder(bool isAscending);

	void		SetPlacementType(PlacementType pt);
	PlacementType GetPlacementType(void) const;

	float		GetSamplingFactor() const;
	void		SetSamplingFactor(float val);

	void		SetBrushHasModified();

// INQUIRY
//
	bool		IsShowingColormap() const;
	bool		IsShowingDerived() const;

protected:

	/// 
	/// Draw the square spiral subwindow visualization
	/// 
	void		DrawSpiral();

	/// 
	/// Draw the circular segment subwindow visualization
	/// 
	void		DrawSegment();

	/// 
	/// Calculate Segments on the Circular window shaped pixel oriented
	/// visualization.
	/// 
	void		CalculateSegments();

	int			FindNearestDim(Vec2 p);

	bool		ShowCaption(grCanvas *canvas, int x, int y);
	void		DrawAxesNameMinMax(grCanvas *canvas);

	void		InputPixelMultiSelect(grCanvas *canvas, int sx, int sy);

   	void		CalcPos(int n, int m, const ARR<double> &data);

protected: 

	MarkerStatus markerStatus;

	int			m_currDim;
	float		m_scale;
	DisplayType	m_display;

	PlacementType m_placementType;
	double		**m_dispPos;

	int			m_sortedDim;
	bool		m_sortAsc;

	bool		m_isShowingColormap;
	int			m_showingDerived;	// 0 = true, 1 = false
	bool		m_isBrushModified;

	float		m_sampleFactor; // [0..1.0f]
	double		m_sampleStart, m_sampleEnd;
	ARR<float>	m_distortions; // [0..1.0f] distortion factor for each dim

	typedef ARR<Image*>		Images;
	Images		m_images; // 

	typedef ARR<Subwindow*> Subwindows;
	Subwindows	m_subwindows;

	typedef ARR<Segment*>	Segments;
	Segments	m_segments;
};

// INLINE METHODS
//

//-----------------------------------------------------------------------------
inline bool CFlatPixel::IsShowingColormap() const
{
	return m_isShowingColormap;
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
inline bool CFlatPixel::IsShowingDerived() const
{
	return m_showingDerived == 0;
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
inline int CFlatPixel::GetSortedDim() const
{
	return m_sortedDim;
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
inline bool CFlatPixel::GetSortDimOrder() const
{
	return m_sortAsc;
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
inline CFlatPixel::PlacementType CFlatPixel::GetPlacementType(void) const
{
	return m_placementType;
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
inline int CFlatPixel::GetSelectedDim() const
{
	return m_currDim;
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
inline float CFlatPixel::GetDimDistortion(int dim) const
{
	return m_distortions[dim];
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
inline float CFlatPixel::GetSamplingFactor() const
{
	return m_sampleFactor;
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
inline void	CFlatPixel::SetBrushHasModified()
{
	m_isBrushModified = true;
}
//-----------------------------------------------------------------------------

#endif // _CFLATPIXEL_H
