#ifndef CFlatScatter_H
#define CFlatScatter_H

#include <xmdv/CFlatDisplay.h>
#include <xmdv/CScatter.h>
#include <xmdv/Vec2.h>

class CFlatScatter : public CFlatDisplay, public CScatter
{
private: 
    ARR<Vec2> poly;

	// Set of static variables to monitor
	// current state of interaction
	int dimx, dimy;
	bool resize_minx, resize_maxx, resize_miny, resize_maxy;

	// PAUL: draw_partial
	// These are for figuring out what fraction of the dataset
	// should be drawn to each scatter plot box
	double xPixelsPerData, yPixelsPerData;
	ARR< ARR<int> > grid_limits;
	int				max_limit;

	// PAUL: reorder_math
	// Saved values for drawing
	ARR<float>	tmpYMAP;

public:
	CFlatScatter(CFlatDisplayInformation* hier_info);
	~CFlatScatter() {}
    //draw single data
    void DrawSingleData(ARR<double> REF data, int datano = 0);
    void DrawSingleDataWithTwoAxes(ARR<double> mean, ARR<double> max, ARR<double> min, int datano, bool unbrushed);
	void ClutterBasedOrder( );		// added by debbie 9/2003

    //event handlers
    void InputMouseMotion(int x, int y);
    void InputButtonPress(int button, int x, int y);
    void InputButtonRelease(int button, int x, int y);

protected:
	bool TestDimensionality();
    //draw axes
    void DrawAxes();
    
    //set color and style for unbrushed data
    void SetUnbrushedColorAndStyle();
    
    //set color and style for average of brushes
    void SetAverageColorAndStyle();
    
    // Draw a single brush.
    void DrawSingleBrush(Brush *brush);

	void RedrawSingleScatter(grCanvas *canvas, int point);
	void DrawScatterAverage(grCanvas *canvas, const ARR<double> &data);
	void DrawScatterBrushBoundRampOutline(grCanvas *canvas, Brush *brush);
	void DrawScatterBrushBoundRamp(grCanvas *canvas, Brush *brush, unsigned long color1, unsigned long color2);
	void DrawScatterBrushBox(grCanvas *canvas, int dimx, int dimy, double data_x, double data_y,
				             double size_x, double size_y);
	void InputScatterResizeBrush(grCanvas *canvas, int sx, int sy);
	void InputScatterResizeBoundRamp(grCanvas *canvas, int sx, int sy);
	void InputScatterMoveBrush(grCanvas *canvas, int sx, int sy);
	void InputScatterMultiSelect(grCanvas *canvas, int sx, int sy);
	void FindResizeCorners(double x,double y);

    // PAUL: scatter_bounds
	virtual void SetParameters();
};


#endif



