///////////////////////////////////////////////////////////// COPYRIGHT //
//
//  XmdvTool - A program for Multivariate Data Display
//
//  Copyright 1994-2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
///////////////////////////////////////////////////////////// COPYRIGHT //

//////////////////////////////////////////////////////////////////////////
/// \file CHierNavhHelperDb.h
/// \brief This file implements the class CHierNavHelper.
/// 
/// This class evaluates the tree_contour required by the class CHierNav 
/// using the values from the data base. This class is instantiated only 
/// when the tool is connected to the data base
///
/// \date 09\25\2003
/// \author Nishant Mehta
//////////////////////////////////////////////////////////////////////////

#ifndef __CHierNavHelperDirectDb_H__
#define __CHierNavHelperDirectDb_H__
#ifdef XMDV_USE_BACKEND
/*============================== INCLUDES ==============================*/

#include <xmdv/CHierNavHelper.h>		// This file has the super class from which this file inherits
#include <string>
//Forward Reference
class Statement;
///
/// This class implements the functions that evaluates the tree_contour
///	from the backend rather than the front end
///
class CHierNavHelperDirectDb : public CHierNavHelper
{
public:		// methods  

	/// Constructor
	CHierNavHelperDirectDb(Statement *statement,
							std::string sourceTableName,
							std::string targetTableName);

	/// Destructor
	~CHierNavHelperDirectDb();

	/// stores the value of the radius of the root node of the tree
	double	root_radius;

	///
	/// Find the radius of the root node
	/// 
	/// \return the radius of the root node
	///
	double	FindRootRadius();

	///
	/// Finds the height of the tree
	/// 
	/// \return The height of the tree
	///
	int		FindTreeHeight();

	///
	/// Populates the tree_contour array
	///
	/// This function evaluates fetches all the different levels from the data base
	/// That is the height of the tree_contour is the same as 
	/// as that of the length of this array. 
	///
	/// \param countour_radius an array that contains all the distinct values 
	///					       for the radius
	///
	void	FindRadiusDistribution(ARR<double> &contour_radius);

	///
	/// Populates the tree_contour
	///
	/// This function evaluates the complete tree_contour. 
	/// That is for each level ot fetches all the nodes at the levels and then
	/// sets the values in the tree_contour[i]. Where i corresponds to the depth 
	/// of the tree you are currently working at
	/// 
	/// \param &tree_contour a 2 dimensional array that stores the complete tree_contour
	/// \param &tree_color a 2 dimensional array that stores the color
	/// \param contour_radius this is used to evaluate the contour, gives all  different level
	///
	void	FindTreeContour(ARR<ARR<double> > &tree_contour,
							ARR<ARR<double> > &tree_color,
							ARR<double> contour_radius);


private:	// members

	int	tree_height;
	//Stores the table name
	std::string tableName;
	//Stores the reference to the statemenet used to execute queries on the data base
	Statement *statement;
	//Stores the source table name
	std::string srcTableName;

};

#endif //XMDV_USE_BACKEND
#endif	// __CHierNavHelperDirectDb_H__
