///////////////////////////////////////////////////////////////////////////////
/// \file CHierPixel.h
/// \brief Declaration of the CHierPixel class
/// \date 5/24/2004
/// \author anilkpatro
///////////////////////////////////////////////////////////////////////////////

#ifndef _CHIERPIXEL_H
#define _CHIERPIXEL_H

// SYSTEM INCLUDES
//
#include <vector>

// PROJECT INCLUDES
//
#include <xmdv/PixelOriented.h>
#include <xmdv/CHierDisplay.h>
#include <xmdv/Vec2.h>
#include <xmdv/BBox2D.h>

// DEFINES
//

// FORWARD REFERENCES
//
class Image;
class Subwindow;

///
/// Hierarchical Pixel Oriented Visualization
///
/// \date 5/24/2004
/// \author anilkpatro
/// \version 1.0
/// \todo // [5/24/2004] Complete class
/// \bug 
///
class CHierPixel : public CHierDisplay, public CPixelOriented
{
public:		//types
//	struct ClusterInfo {
//		Cluster *cluster;
//		bool isBrushed;
//		ClusterInfo() {
//			cluster = NULL; isBrushed = false;
//		}
//		ClusterInfo(const Cluster &c, bool b) { 
//			Set(c, b);
//		}
//		void Set(const Cluster &c, bool b) {
//			cluster = const_cast<Cluster*>(&c); 
//			isBrushed = b;
//		}
//	};
//	typedef ARR<ClusterInfo> ClusterInfos;

public:		// methods

// LIFECYCLE
//	
	///
	/// Constructor
	/// 
	CHierPixel(CHierDisplayInformation* hier_info);

private:
	/// 
	/// Copy constructor
	/// 
	/// \param from the value to copy to this object
	/// 
	CHierPixel(const CHierPixel& from);
public:
	/// 
	/// Destructor
	/// 
	virtual ~CHierPixel();

// OPERATORS
//
private:
	/// 
	/// Assignment operator
	/// 
	/// \param from the value to assign to this object
	/// \return reference to this object
	CHierPixel& operator = (const CHierPixel& from);

public:
// OPERATIONS
//
	void		PrepareCanvas();
	void		Draw();
	void		DrawAxes();
	void		DrawSingleCluster( const Cluster &c, bool brushed );

	//event handlers
	void		InputMouseMotion(int x, int y);
	void		InputButtonPress(int button, int x, int y);
	void		InputButtonRelease(int button, int x, int y);

protected:	// methods

	void		SetParameters();

	void		DrawSpiral();
	void		DrawAllClusters();
	void		TraceDrawAllClusters(const Cluster &node);

	int			FindNearestDim(Vec2 p);
	void		DrawAxesNameMinMax(grCanvas *canvas);

	void		ShowCaption(grCanvas *canvas, int x, int y);

	void		ComputeQueryDist(int sortedDim = 0, bool isAscending = true);
	void		CountUnbrushedClusters( const Cluster &node );
	void		CountBrushedClusters( const Cluster &node );

private:	// members
	int			m_currDim;
	int			m_sortedDim;
	bool		m_isAscending;
	bool		m_zoomDim;
	bool		m_isPanning;

	int			m_NumUnbrushedClusters, 
				m_NumBrushedClusters,
				m_NumClusters;
	int			m_currCluster;
	ARR<double> hier_combined_distances;

	MarkerStatus	m_markerStatus;

	typedef ARR<Image*> Images;
	Images		m_images;
	typedef ARR<Subwindow*> Subwindows;
	Subwindows	m_subwindows;
	//Zaixian start
	bool		m_isShowingColormap;
	int			m_showingDerived;	// 0 = true, 1 = false
	ARR<float>	m_distortions; // [0..1.0f] distortion factor for each dim
	bool m_hasDistortionChanged;
	bool m_reset;
	//Zaixian end

//Added by Zaixian start
public:
	//Functions to setup the pixel-oriented display
	void SetSortedDim(int dim) ;
	void SetSortDimOrder(bool isAscending) ;
	void SelectDim(int dim) ;
	void SetDimDistortion(int dim, float dist) ;
	void ShowColormap(bool bShow) ;
	void ShowDerived(bool bShow) ;
	void SetPlacementType(PlacementType pt) ;
	void SetSamplingFactor(float val) ;
	float GetDimDistortion(int dim) const;
	int GetSelectedDim() const;
//Zaixian end
};

// INLINE METHODS
//

#endif	// _CHIERPIXEL_H
