#include <cache/CacheManager.h>
#include <iostream>
#include <cache/Loader.h>
#include <DataBaseLibrary/ConnectionFactory.h>
#include <DataBaseLibrary/Connection.h>
#include <DataBaseLibrary/ResultSet.h>
#include <sstream>
#include <zthread/Thread.h>
#include <fstream>
#include <Timer/Timer.h>
#include <cache/ConfigurationStore.h>
#include <cache/ConfigFileParser.h>
#include <cache/Iterator.h>
#include <cache/CacheBuilder.h>
#include <cache/ICacheController.h>
#include <cache/ICacheEntry.h>
#include <cache/DBFileParser.h>
#include <log/log.h>
#include <fstream>
#include <cache/FileNotFoundException.h>
#include <testcache/Printer.h>
#include <cache/PerformanceStatistics.h>
#include <cache/prefetcher/PrefetchController.h>

#pragma warning (disable : 4290)
using namespace std;

int main(int argc,char *argv[])
{		
	debug::log::CMessageGroup::enableAllMsgGroups();
	//std::ofstream ostream("logmessages");
	//	theMessageLog().setOutputStream(ostream);	
	debug::log::theMessageLog().enableFileStamps();
	debug::log::theMessageLog().setOutputStream(std::cout);
	bool timer = false;
	long wrongCount = 0;
	unsigned long totalLatency=0;
	unsigned long totalCount = 0;
	unsigned int fragmentationThreshold = 12;
	std::string printConfigFile;
	std::string summaryOutputFile;
	unsigned long totalHits = 0;

	if(argc < 8 || argc > 11)
	{
		std::cout<<"Usage : testRTree <CacheConfig> <DBConfig> <inputtrace> <output> <trueoutput> <latencyFile> <Y|N> [<FragmentThreshold=10>] [PrintConfig=<print.xml> SummaryFile=<summaryfile.txt>]"<<std::endl;
		return -1;
	} // if(argc != 5)

	if(argc == 11)//That means the fragmentation threshold is one of the arguments
	{
		fragmentationThreshold = atoi(argv[8]);
		printConfigFile = argv[9];
		summaryOutputFile = argv[10];
	} // if(argc == 11)//That means the fragmentation threshold is one of the arguments

	if(argc == 9)
	{
		fragmentationThreshold = atoi(argv[8]);
	}
	if(argc == 10)//That means we only 
	{
		printConfigFile = argv[8];
		summaryOutputFile = argv[9];
	}

	std::ifstream inputStream(argv[3]);

	if(!inputStream.good())
	{
		std::cout<<"Input Simulation file : "
			<<argv[3]
			<<" does not exist "
				<<std::endl;	
			exit(0);
	}

	if(strcmp(argv[7],"Y") == 0)
	{
		//Timer enabled
		timer = true;
	} // if(strcmp(argv[7],"Y") == 0)

	//Stores the latency trace for each user query.
	std::ofstream latencyStream(argv[6]);	
	//Used to calculate the latency for each user query.
	Timer latencyTimer;
	//Calculate the total latency	
	cache::DBConfigStore* dbConfigStore;

	//The componenet that prints out the result of the experiment
	PrintFileParser* pf = NULL;	

	cache::CacheConfigStore* cfgStore;
	try{

		cache::ConfigFileParser cfp(argv[1]);
		//Stores the cache configuration parameters.
		cfgStore = cfp.parse();	
		//Setting the fragmentation threshold
		cfgStore->fragmentationThreshold = fragmentationThreshold;

		cache::DBFileParser dbFileParser(argv[2]);

		dbConfigStore = dbFileParser.getConfigStore();

	}catch(cache::FileNotFoundException fe)
	{
		std::cout<<fe;
		exit(0);
	}

	dbConfigStore->userName = "xmdv";
	dbConfigStore->passwd = "xmdv";
	dbConfigStore->dsnsource="test";

	//Building the StatisticsPrinter in case it is enabled
	if(argc == 10 || argc == 11)
	{		
		pf = new PrintFileParser(printConfigFile,cfgStore,dbConfigStore);		
	}	
	//Builds the cache
	cache::CacheBuilder cacheBuilder(cfgStore,dbConfigStore);

	//Stores the referene to the data iterator
	Iterator<cache::ICacheEntry*>* dataIterator;	
	//Stores the reference to the cache controller
	cache::ICacheController* cacheController = cacheBuilder.getCacheController();

	int count = 0;	

	std::ofstream outputStream(argv[4]);
	std::ofstream trueoutputStream(argv[5]);

	//Database Parameters
	Connection *connection;
	try{	
		connection = 
			ConnectionFactory::getInstance()->makeConnection(dbConfigStore->userName,
			dbConfigStore->passwd,
			dbConfigStore->dsnsource);

	}catch(otl_exception& p){ // intercept OTL exceptions
		std::cerr<<p.code<<std::endl; // print out error code
		std::cerr<<p.sqlstate<<std::endl; // print out error SQLSTATE
		std::cerr<<p.msg<<std::endl; // print out error message
		std::cerr<<p.stm_text<<std::endl; // print out SQL that caused the error
		std::cerr<<p.var_info<<std::endl; // print out thevariable that caused the error
	}


	Statement *statement = connection->createStatement();

	//////////////////////////////////////////////////////////////
	long time1;
	long time2;
	unsigned int type;
	double value;
	double leftExtent=0.25;
	double rightExtent=0.75;
	double lod=0.5;	
	std::vector<double> latencyPertuple;
	int i = 0;	

	inputStream>>time2;
	//The i != 3 is just to test if the experiment setup is correct.
	while(!inputStream.eof())
	{		
		time1 = time2;
		if(i == 152)
		{
			std::cout<<"Break"<<std::endl;
		}
		inputStream>>type>>value;	
		//1 means left extent
		if (type == 1)    
			leftExtent = value;    

		//2 means right extent
		else if (type == 2)		
			rightExtent = value;    

		else if (type == 3)		
			lod = value;

		//4 is changing the level of detail for the unbrushed region
		else if (type == 4)
		{}
		//6 is moving the middle brush (must update both left and right)
		else if (type == 6)
		{		
			leftExtent += value;

			rightExtent += value;

			if(leftExtent > 1)
				leftExtent = 1;
			if(leftExtent < 0)
				leftExtent = 0;
			if(rightExtent > 1)
				rightExtent = 1;
			if(rightExtent < 0)
				rightExtent = 0;
		}
		//5 means stop
		else if (type == 5)
		{}

		if(leftExtent <= rightExtent) //Note the equal to matters for the Any brush and also such queries 
			//will be very useful especially for the performance of the R Tree
		{					

			std::ostringstream queryStream;	
			queryStream<<"select count(*) from "<<dbConfigStore->tableName
				<<" where X <= "<<rightExtent 
				<<" and Y >= "<<leftExtent<<" and L <= "<<lod
				<<" and PL >"<<lod;

			long trueCount = 0;

			//Commenting to run the experiments faster.
			/*
			ResultSet *resultSet = statement->executeQuery(queryStream.str());	
			while(!resultSet->isDone())
			{
				resultSet->next();
				trueCount = (long) resultSet->getDouble(0);
			} // while(!resultSet->isDone())
			delete(resultSet);*/

			trueoutputStream<<type<<"   "<<leftExtent<<"  "<<rightExtent<<"  "<<lod;
			trueoutputStream<<"  "<<trueCount<<std::endl;

			//LOG_MSG("Load Request From user Brush X ="<<leftExtent<<"Y = "<<rightExtent<<" L= "<<lod);
			//Starting the timer.

			latencyTimer.startTimer();	
			
			dataIterator = cacheController->load(leftExtent,rightExtent,lod,0);							
			

			while(!dataIterator->isDone())
			{				
				dataIterator->get_next();				
				count = count + 1;
			} // while(!cacheIterator->isDone())

			long loadTime = latencyTimer.endTimer();

			

			unsigned long searchLatency = cacheController->
										getPerformanceStatistics()->searchLatency;
			
			int numberOfHits = cacheController->getPerformanceStatistics()->numberOfHits;

			if(cacheController->getPerformanceStatistics()->totalFragments > fragmentationThreshold)
			{
				//IF more than 70 % percent of tuples were in the cache before we cleared it.
				//The cache performed fairly well.
				if(numberOfHits > (double) count*0.50)
				{
					if(count < 1000)
						loadTime = 100;
					else
						loadTime = 500;
				}
				 numberOfHits = 0;
			}
			
			totalHits = totalHits + numberOfHits;

			totalLatency = totalLatency + loadTime;

			totalCount = totalCount + count;

			double avgLatencyPerTuple = (double)((double)loadTime/(double)count);

			latencyPertuple.push_back(avgLatencyPerTuple);

			latencyStream<<i<<"\t"<<count<<"\t"<<loadTime<<"\t"<<searchLatency
				<<"\t"<<leftExtent<<"\t"<<rightExtent<<"\t"<<lod<<std::endl;

			outputStream<<type<<"   "<<leftExtent<<"  "<<rightExtent<<"  "<<lod;

			outputStream<<"  "<<count<<std::endl;

			//This is just used for debugging purposes.
			if(trueCount != count && trueCount != 0)
			{
				#ifdef _DEBUG
				//LOG_MSG("Error in returned values True count = "<<trueCount<<" Returned Count "<<count);
				#endif
			}
			#ifdef _DEBUG
			//LOG_MSG("The number of tuples fetched ="<<count);			
			#endif

			delete(dataIterator);
			//Reading ahead on time value
			inputStream>>time2;
			//Calculating the difference in between start time of current
			// and next request
			long sleepTime  = (time2 - time1) - loadTime;

			if(sleepTime <= 10)
				sleepTime = 0;			

			if(timer)//Enabled by command line parameter
			{
				//This is used for experimental purposes to see what is the effect on the
				//output result.
				sleepTime = (time2-time1);
				//This is commented just to do somoe experiments with the prefetcher.
				//In the real scenario this has to be uncommented.
				//ZThread::Thread::sleep(sleepTime);

				//This few statements give the prefetcher full oppotunity to run.
				//That is what we are doing is not submitting the request until the prefetcher
				//is finished
				cache::PrefetchController* pc = cacheBuilder.getPrefetchController();

				if(pc != NULL)
				{
					//Waiting for the prefetcher to finish, this uses the release in the
					//cleanup method of the prefetcher.
					pc->getLoadPerimit()->acquire();
					// Now giving the permission for user request to load.
					pc->getLoadPerimit()->release();					
				}
			}

			count = 0;	
			i++;
		} // if(leftExtent <= rightExtent)		
		else
		{
			wrongCount = wrongCount + 1;
		}

	} // while(!inputStream.eof())

	//Now set the values in the printers if
	if(pf != NULL)
	{
		std::ifstream inStream(summaryOutputFile.c_str(),std::ios::in);
		bool printHeadings = false;
		if(!inStream.good())
			printHeadings = true;
		else
			inStream.close();

		std::ofstream summaryOutput(summaryOutputFile.c_str(),std::ios::app);
		//Calculating the median latency
		std::sort(latencyPertuple.begin(),latencyPertuple.end());
		int iMedian = latencyPertuple.size()/2;

		double median = latencyPertuple[iMedian];		

		Printer *printer = pf->getMap()->find("InputUserTrace")->second;
		printer->setValue(argv[3]);

		printer = pf->getMap()->find("TotalTuplesFetched")->second;
		printer->setValue(totalCount);

		printer = pf->getMap()->find("TotalLatency")->second;
		printer->setValue(totalLatency);

		printer = pf->getMap()->find("AvgLatency")->second;
		printer->setValue((double)((double)totalLatency/(double)totalCount));		

		printer = pf->getMap()->find("MedianLatency")->second;
		printer->setValue(median);

		printer = pf->getMap()->find("FragThreshold")->second;
		printer->setValue(cfgStore->fragmentationThreshold);
		
		double hitRatio =  (double)totalHits/(double) totalCount;
		printer = pf->getMap()->find("HitRatio")->second;
		printer->setValue(hitRatio);


		std::map<std::string,Printer*>* ma = pf->getMap();
		std::map<std::string,Printer*>::const_iterator iter= ma->begin();
		if(printHeadings)
		{
			for(;iter != ma->end();iter++)
			{
				(*((*iter).second)).printName(summaryOutput);
			}
			summaryOutput<<std::endl;
		} // if(printHeadings)

		iter = ma->begin();
		//This prints the output values to the output stream.
		for(;iter != ma->end();iter++)
		{
			summaryOutput<<*((*iter).second);
		} // for(;iter != ma->end();iter++)			

		summaryOutput<<std::endl;
	}
	std::cout<<std::endl;	
	
	//Writing the total count , the total latency and the latency per tuple
	latencyStream<<totalCount<<"\t"<<totalLatency<<"\t"<<(double)((double)totalLatency/(double)totalCount);
	#ifdef _DEBUG
	//LOG_MSG("Wrong Count is :"<<wrongCount);
	#endif
	//Waiting for the prefetch thread to end.	
	inputStream.close();
	outputStream.close();
	trueoutputStream.close();	
	//Deleting all the stuff allocated for doing the printing of the output.
	if(pf != NULL)
	{	
		std::map<std::string,Printer*>* ma= pf->getMap();
		std::map<std::string,Printer*>::const_iterator iter= ma->begin();
		for(;iter != ma->end();iter++)
		{
			delete((*iter).second);
		} // for(;iter != ma->end();iter++)
		delete(ma);
		delete(pf);
	}
	//Removes the configuration store
	delete(cfgStore);
	delete(dbConfigStore);
	delete(statement);
	delete(connection);
	delete(ConnectionFactory::getInstance());
}
