#include <xmdv/ogl.h>
#include <xmdv/CDisplay.h>
#include <xmdv/XmdvTool.h>
#include <xmdv/colormap.h>
#include <xmdv/magnify.h>
#include <xmdv/brush_oper.h>
#include <xmdv/okcdata.h>


CDisplay::CDisplay()
{
	xor_needs_undo = false;
	input_mode = NONE;
	canvas_mode = DISP_2D;
}


CDisplay::~CDisplay()
{
}

//some display requires minimum dimensionality, reture true if there is enough dimensions.
bool CDisplay::TestDimensionality()  
{
	return true;
}


void CDisplay::CanvasDraw()
{
	if ( ! g_globals.dataset || g_globals.dataset->datafile == NULL )
		return;
	if ( TestDimensionality() ) {
//		PrepareCanvas();
		Draw();
		//DrawMagnify(0.5, -0.5);
	}
}


void CDisplay::CanvasInputKeyPress(int key)
{
	if ( ! g_globals.dataset )
		return;
	if ( TestDimensionality() )
		InputKeyPress(key);
}


void CDisplay::CanvasInputKeyRelease(int key)
{
	if ( ! g_globals.dataset )
		return;
	if ( TestDimensionality() )
		InputKeyRelease(key);
}


void CDisplay::CanvasInputMouseMotion(int x, int y)
{
	if ( ! g_globals.dataset )
		return;
	if ( TestDimensionality() )
		InputMouseMotion(x, y);
}


void CDisplay::CanvasInputButtonPress(int button, int x, int y)
{
	if ( ! g_globals.dataset )
		return;
	if ( TestDimensionality() )
		InputButtonPress(button, x, y);
}


void CDisplay::CanvasInputButtonRelease(int button, int x, int y)
{
	if ( ! g_globals.dataset )
		return;
	if ( TestDimensionality() )
		InputButtonRelease(button, x, y);
}

void CDisplay::DrawMagnify(float x, float y)
{
//	canvas->SetTopLeftOrigin();
//	grCanvas::grResizeMainCanvas(canvas);
//
//	canvas->SetBackground(interface_color[BG]);

	glClear(GL_COLOR_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);    // Clear The Screen And The Depth Buffer and Stencil Buffers
	glPushMatrix();
	glLoadIdentity();                         // Reset The View

	// Create a square area in the stencil buffer
	glPolygonMode(GL_FRONT, GL_FILL);
	glPolygonMode(GL_BACK, GL_FILL);
	glStencilFunc(GL_ALWAYS, 1, 1);
	glStencilOp(GL_REPLACE, GL_REPLACE, GL_REPLACE);
	glBegin(GL_QUADS); {
		glVertex3f(-0.1f + x, -0.1f - y,  0.0);
		glVertex3f( 0.1f + x, -0.1f - y,  0.0);
		glVertex3f( 0.1f + x,  0.1f - y,  0.0);
		glVertex3f(-0.1f + x,  0.1f - y,  0.0);
	} 
	glEnd();


	// clear buffers and draw screen and draw only in that square area
	glClear(GL_COLOR_BUFFER_BIT);    // Clear The Screen And The Depth Buffer
	glStencilFunc(GL_EQUAL, 1, 1);
	glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);

	glTranslatef(0.5f, 0.5f, 0);
	glScalef(2.f,2.f,2.f);
	glTranslatef(-0.5f, -0.5f, 0);
	Draw();
	glScalef(.5f,.5f,.5f);

	// draw a square border so that you can see where the square is
	glPolygonMode(GL_FRONT, GL_LINE);
	glPolygonMode(GL_BACK, GL_LINE);
	glLineWidth(2);         // set a thick line for square border
	RGBt fgcol(interface_color[TEXT]);
	glColor3f((GLfloat)fgcol.R, (GLfloat)fgcol.G, (GLfloat)fgcol.B);     // draw a black boder
	glBegin(GL_QUADS); {      // draw the square borders
		glVertex3f(-0.1f + x, -0.1f - y,  0.0);
		glVertex3f( 0.1f + x, -0.1f - y,  0.0);
		glVertex3f( 0.1f + x,  0.1f - y,  0.0);
		glVertex3f(-0.1f + x,  0.1f - y,  0.0);
	}
	glEnd();
	glLineWidth(1);         // restore line thickness
	glPolygonMode(GL_FRONT, GL_FILL);
	glPolygonMode(GL_BACK, GL_FILL);

	glPopMatrix();

	// now draw in the area that is not in the square (Not equal to sqaure area)
	glStencilFunc(GL_NOTEQUAL, 1, 1);
	glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
	Draw();
}

void CDisplay::Reshape()
{
    canvas->grResizeMainCanvas(canvas);
}


void CDisplay::Draw()
{
}



void CDisplay::InputKeyPress(int key)
{
//    appCanvasInputKeyPress(canvas, key);
}


void CDisplay::InputKeyRelease(int key)
{
//    appCanvasInputKeyRelease(canvas, key);
}


void CDisplay::InputMouseMotion(int x, int y)
{
}


void CDisplay::InputButtonPress(int button, int x, int y)
{
}


void CDisplay::InputButtonRelease(int button, int x, int y)
{
}


void CDisplay::SetCanvas(grCanvas *canvas)
{
    this->canvas = canvas;
}


void CDisplay::PrepareCanvas()
{
	if (canvas_mode == DISP_2D)
		canvas->SetMode(grCanvas::CM_2D);
	else
		canvas->SetMode(grCanvas::CM_3D);
    canvas->SetTopLeftOrigin();
    grCanvas::grResizeMainCanvas(canvas);

    canvas->SetBackground(interface_color[BG]);
    canvas->Clear();
}

void CDisplay::ClutterBasedOrder()		//added by debbie 9/2003
{
}
