///////////////////////////////////////////////////////////////////////////////
/// \file CFlatPixel.cpp
/// \brief Draw data using a pixel oriented visualization
/// \date 5/6/2004
/// \author Anilkumar Patro
///////////////////////////////////////////////////////////////////////////////

// SYSTEM INCLUDES
//
#include <cstdlib>
#include <cstring>
#include <sstream>
using namespace std;

// PROJECT INCLUDES
//
#include <xmdv/CFlatPixel.h>
#include <xmdv/ogl.h>
#include <xmdv/XmdvTool.h>
#include <xmdv/colormap.h>
#include <xmdv/Image.h>
#include <xmdv/colorramp.h>
#include <xmdv/Subwindow.h>
#include <xmdv/ColorMapEditor.h>
#include <xmdv/multi_select.h>
//#include <xmdv/FlatPixelSettings.h>
#include <xmdv/Segment.h>

#include <xmdv/mds/Multidimensional_scaling.h>

// FORWARD DECLARATION
//

namespace
{

bool isPanning;
double resize_y;
//bool zoomDim;
bool hasDistortionChanged;
bool hasPlacementChanged;
bool reset;
INPUT_MODE mode;

//CFlatPixelSettings *settingsWindow	= 0;
//PlusMinusPanel *plusminusPanel	= 0;

};

//-----------------------------------------------------------------------------
CFlatPixel::CFlatPixel(CFlatDisplayInformation *flat_info)
	: CFlatDisplay(flat_info), m_currDim(0)
{
	m_display = CFlatPixel::PDT_SPIRAL;
	isPanning = false;
	markerStatus = MST_BLANK;
	//	zoomDim = false;
	m_isBrushModified = true;
	m_sortedDim = 0; m_sortAsc = true;
	mode = NONE;
	reset = true;
	m_sampleFactor = 1.0f;
	m_sampleStart = 0; m_sampleEnd = 1.0f;
	hasDistortionChanged = true;
	hasPlacementChanged = true;

	m_placementType = PPT_NORMAL;
	m_dispPos = NULL;

	m_isShowingColormap = true;
	m_showingDerived = 0;
}
//-----------------------------------------------------------------------------

CFlatPixel::CFlatPixel(const CFlatPixel &c)
{
	c;
}

//-----------------------------------------------------------------------------
CFlatPixel::~CFlatPixel()
{
	FreeSpace();
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CFlatPixel::DrawSpiral()
{
	// figure out the size of each subwindow in terms of pixels
	int width, height; canvas->GetSize(&width, &height);

	static int size				= 0;
	static float xsize = 0, ysize = 0;
	static float linex			= 0;
	static int numSubWindInRow	= 0;

	// allocate the images to be used
	if (m_images.Len() != g_globals.dataset->dims + 1 ||
		m_images[0]->Width() != size + 1 ||
		reset)
	{
		size = (int) 
			   ((sqrt(g_globals.dataset->data_size * m_sampleFactor))+0.5f)+1;
		xsize = size / (float) width;
		ysize = size / (float) height;

		m_currDim = m_showingDerived;
		m_sortedDim = m_showingDerived;
		m_sortAsc = true;
		m_isBrushModified = false;
		hasDistortionChanged = true;
		hasPlacementChanged = true;

		// delete things before allocating new ones
		for (int i = 0; i < m_images.Len(); ++i)
		{
			delete (Image *) m_images[i];
			delete (Subwindow *) m_subwindows[i];
			m_images[i] = NULL; m_subwindows[i] = NULL;
		}
		m_images.Clear(); m_subwindows.Clear();
		m_images.SetLen(g_globals.dataset->dims + 1);
		m_subwindows.SetLen(g_globals.dataset->dims + 1);
		canvas->boundingBox.Invalidate();
		m_distortions.Clear();
		m_distortions.Set(g_globals.dataset->dims + 1, 0.0f);

		// allocate new ones
		// figure out the number of subwindows that would fit on each row
		for (numSubWindInRow = 1; /*empty*/ ; ++numSubWindInRow)
		{
			if (numSubWindInRow * numSubWindInRow >= g_globals.dataset->dims)
				break;
		}

		for (int dim = 0; dim < g_globals.dataset->dims + 1; ++dim)
		{
			Image *img	= new Image(size + 1, size + 1);
			m_images[dim] = img;

			Subwindow *subwnd	= new Subwindow(dim);
			m_subwindows[dim] = subwnd;
		}
		//if (settingsWindow && settingsWindow->isVisible())
		//	settingsWindow->Reset();
		//Zaixian Start
		//Tcl_Eval(Togl_Interp(canvas->GetWidget()),
		//	"::.flatPixelSettings::SetupFlatPixelSettingsDialog");
		//Zaixian End
	}

	// figure out where to place the subwindow
	float prev_width = xsize, prev_height = ysize;
	int k	= -1;
	if (reset || hasPlacementChanged || hasDistortionChanged)
	{
		canvas->boundingBox.Invalidate();
		for (int dim = 0; dim < g_globals.dataset->dims + 1; ++dim)
		{
			Subwindow *subwnd	= m_subwindows[dim];
			const float xs		= (xsize + 5.0f / width);
			float x = 0, y = 0;
			if (m_placementType == PPT_NORMAL)
			{
				if (dim > 0)
				{
					x = (((dim - 1) % numSubWindInRow) * xs) + xs + 0.04f;
					y = ((((dim - 1) % numSubWindInRow == 0) ? ++k : k) * 
						(ysize + 18.0f / height));
				}
			} else
			{
				x = (float) m_dispPos[0][dim];
				y = (float) m_dispPos[1][dim];
			}

			prev_width = xsize + m_distortions[dim] * width / height;
			prev_height = ysize + m_distortions[dim] * width / height;
			if (dim == 0)
				linex = prev_width + 0.02f;

			subwnd->SetWindow(x, y, prev_width, prev_height);
			canvas->boundingBox.Add(subwnd->GetBoundingBox());
		}
		reset = false;
		hasDistortionChanged = hasPlacementChanged = false;
	}

	// here is the actual painting...
	Vec2 font_size		= canvas->GetDataFontSize("W");

	// for all subwindows
	ARR<int> priority	(g_globals.dataset->dims + 1);
	int dim;
	for (dim = m_showingDerived; dim < m_currDim; ++dim)
		priority[dim] = dim;
	for (dim = m_currDim + 1; dim < g_globals.dataset->dims + 1; ++dim)
		priority[dim - 1] = dim;
	priority[g_globals.dataset->dims] = m_currDim;

	dim = priority[0];
	for (int i = m_showingDerived; i < g_globals.dataset->dims + 1; ++i)
	{
		dim = priority[i];
		Subwindow *subwnd	= m_subwindows[dim];

		double max_str_size	= subwnd->GetWidth() - font_size.X*5;
		if (dim == 0)
		{
			canvas->DrawConfinedString(subwnd->GetX(),
									   subwnd->GetY() - (2.0f / width),
									   "Combined Distance", max_str_size);
			if (m_placementType == PPT_NORMAL)
			{
				RGBt color	(interface_color[TEXT]);
				glColor3f((float) color.R, (float) color.G, (float) color.B);
				glLineWidth(2.0f);
				canvas->DrawLine(linex, 1.04f, linex, -.04f);
				glLineWidth(1.0f);
			}
		} else
			canvas->DrawConfinedString(subwnd->GetX(),
									   subwnd->GetY() - (2.0f / width),
									   g_globals.dataset->names[dim - 1],
									   max_str_size);

		// paint the subwindow
		subwnd->Paint(m_images[dim], flat_query, m_sampleStart, m_sampleEnd, dim == m_currDim);
	}
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CFlatPixel::Draw()
{
	if (!g_globals.dataset)
		return FreeSpace();

	//set canvas background, clear canvas
	PrepareCanvas();

	if (TestDimensionality()) //some display requires minimum dimensionality
	{
		//set parameters used for drawing
		SetParameters();

		//draw axes
		DrawAxes();

		//draw other things such as lines links star glyphs...
		DrawOthers();

		//free space used for drawing
		//		FreeSpace();
	}
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CFlatPixel::DrawAxes()
{
	if (m_isBrushModified)
	{
		ComputeQueryDist(m_sortedDim, m_sortAsc);
		if (m_placementType == PPT_MDS)
		{
			CalcPos(g_globals.dataset->data_size, g_globals.dataset->dims + 1,
					flat_query.distances);
			hasPlacementChanged = true;
		}
		m_isBrushModified = false;
	}

	// draw the pixels
	switch (m_display)
	{
	case CFlatPixel::PDT_SPIRAL:
		DrawSpiral(); break;
	case CFlatPixel::PDT_SEGMENTS:
		DrawSegment(); break;
	default:
		assert(true);
	}

	if (!m_isShowingColormap)
		return CFlatDisplay::DrawAxes();

	glPushMatrix();
	glLoadIdentity();

	// draw background
	glLineWidth(45.0f);
	RGBt color	(interface_color[BG]);
	glColor3f((float) color.R, (float) color.G, (float) color.B);
	glBegin(GL_LINE_STRIP); {
		glVertex2f(1.015f, -0.04f);
		glVertex2f(1.015f, 1.04f);
	} glEnd();

	// get brush extents
	if (m_currDim == 0)
	{
		QueryDist &query_dist	= flat_query;
		double bb				= (query_dist.brush_level - query_dist.dist_min) /
								  (query_dist.dist_max - query_dist.dist_min);

		g_globals.pixelBrushedCMap->DrawColorBar(1.0f, 1.0f, 1.0f,
												 (float) YMAP(bb), 20.0f);
		g_globals.pixelUnbrushedCMap->DrawColorBar(1.0f, (float) YMAP(bb), 1.0f,
												   0.0f, 20.0f);
	} else
	{
		double brushMax, brushMin;
		GetNormalizedBrushExtents(&g_globals.brushes[0], m_currDim - 1,
								  brushMax, brushMin);
		double brushCenter	= ((brushMax - brushMin) * 0.5) + brushMin;

		// draw the colormap
		g_globals.pixelBrushedCMap->DrawColorBar(1.0f,
												 (float) YMAP(brushCenter),
												 1.0f, (float) YMAP(brushMax),
												 20.0f);
		g_globals.pixelBrushedCMap->DrawColorBar(1.0f,
												 (float) YMAP(brushCenter),
												 1.0f, (float) YMAP(brushMin),
												 20.0f);
		g_globals.pixelUnbrushedCMap->DrawColorBar(1.0f, (float) YMAP(brushMin),
												   1.0f, 1.0f, 20.0f);
		g_globals.pixelUnbrushedCMap->DrawColorBar(1.0f, (float) YMAP(brushMax),
												   1.0f, 0.0f, 20.0f);
		glLineWidth(1.0f);

		// draw markers
		glBegin(GL_TRIANGLES); {
			if (markerStatus == MST_ON_MIN || markerStatus == MST_DRAG_MIN)
				glColor3f(1.0f, 0, 0);
			else
			{
				RGBt color	(interface_color[TEXT]);
				glColor3f((float) color.R, (float) color.G, (float) color.B);
			}
			glVertex2f(1.02f, (float) YMAP(brushMin));
			glVertex2f(1.03f, (float) YMAP(brushMin) + 0.01f);
			glVertex2f(1.03f, (float) YMAP(brushMin) - 0.01f);

			if (markerStatus == MST_ON_MAX || markerStatus == MST_DRAG_MAX)
				glColor3f(1.0f, 0, 0);
			else
			{
				RGBt color	(interface_color[TEXT]);
				glColor3f((float) color.R, (float) color.G, (float) color.B);
			}
			glVertex2f(1.02f, (float) YMAP(brushMax));
			glVertex2f(1.03f, (float) YMAP(brushMax) + 0.01f);
			glVertex2f(1.03f, (float) YMAP(brushMax) - 0.01f);
		} glEnd();


		// draw labels
		if (!isPanning)
			DrawAxesNameMinMax(canvas);
	}

	glPopMatrix();

	// call base class implementation
	CFlatDisplay::DrawAxes();
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CFlatPixel::DrawSegment()
{
	int width, height; canvas->GetSize(&width, &height);
	m_distortions.Clear();
	m_distortions.Set(g_globals.dataset->dims + 1, 0.0f);

	// allocate the image
	if (m_images.Len() != 1)
	{
		m_images.SetLen(1);
		Image *img	= new Image(1024, 1024);
		m_images[0] = (img);
		img->Clear(255, 255, 255);
	}

	// calculate the number of segments and their placements
	CalculateSegments();

	Image *img	= m_images[0];

	// for all segments
	for (int dim = 0; dim < g_globals.dataset->dims+1; ++dim)
	{
		// draw the segment on the on the image
//		m_segments[dim]->Paint(img, m_sampleFactor, dim == m_currDim);
	}

	// display the segments
	float xsize = 1024 / (float) width, ysize = 1024 / (float) height;
	float x = 0, y = 0;
	img->CreateTexture();
	img->Display(x, y, x+xsize, y+ysize);
	img->DeleteTexture();
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CFlatPixel::DrawOthers()
{
	/*
	static bool init= false;
	if (!init)
	{
		settingsWindow = new CFlatPixelSettings(Togl_Interp(canvas->GetWidget()));
		plusminusPanel = new PlusMinusPanel(Gooey::Core::Rectangle(0, 0, 20, 20),
											settingsWindow);

		// add the windows to the manager
		Gooey::WindowManager::instance()
		.addWindow(plusminusPanel)
		.addWindow(settingsWindow)
		;
		settingsWindow->hide();

		init = true;
	}

	// this must be called once per frame
	Gooey::WindowManager::instance().update();

	if (settingsWindow->isVisible())
	{
		int width, height;
		canvas->GetSize(&width, &height);
		settingsWindow->Update(width, height);
	}
	*/
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CFlatPixel::ShowColormap(bool bShow)
{
	m_isShowingColormap = bShow;
	canvas->Refresh();
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CFlatPixel::ShowDerived(bool bShow)
{
	m_showingDerived = (bShow == true) ? 0 : 1;
	if (m_currDim == 0 && m_showingDerived == 1)
		SelectDim(1);
	canvas->Refresh();
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CFlatPixel::InputMouseMotion(int sx, int sy)
{
	if (!g_globals.dataset)
		return;

	ShowCaption(canvas, sx, sy);

	if (mode == MULTI_SELECT)
	{
		InputPixelMultiSelect(canvas, sx, sy);
		return;
	}

	if (canvas->button[2] == true)
	{
		appCanvasMousePan(canvas);
		return;
	}

	if (!m_isShowingColormap)
		return appRefreshAllCanvas(Togl_Interp(canvas->GetWidget()));

	if (m_currDim == 0)
		return;
	int dim			= m_currDim - 1;

	// get brush extents
	Vec2 p			= MapScreenToCMapData(Vec2(sx, sy));

	// Clamp y coordinate to within dimension limits
	double y		= YINVMAP(dim, p.Y);
	double move_y	= MAX2(y, g_globals.dataset->dim_min[dim]);
	move_y = MIN2(move_y, g_globals.dataset->dim_max[dim]);

	if (markerStatus == MST_RESIZE)
	{
		double diff_y	= y - resize_y;
		double sz		= g_globals.brushes[0].Size(dim);
		double pos		= g_globals.brushes[0].Pos(dim);
		double newsz;
		if (diff_y > 0)
			newsz = sz * 1.1;
		else
			newsz = sz * 0.9;

		double hy	= MIN2(pos + newsz*.5, g_globals.dataset->dim_max[dim]);
		double ly	= MAX2(pos - newsz*.5, g_globals.dataset->dim_min[dim]);
		g_globals.brushes[0].SetMinMax(dim, ly, hy);
		m_isBrushModified = true;
		appBrushBoundaryCaption(canvas, dim);
		resize_y = y;
	} else if (markerStatus == MST_TRANSLATE)
	{
		double trans_y	= MIN2(y,
							   g_globals.dataset->dim_max[dim] -
							   g_globals.brushes[0].Size(dim) * .5);
		trans_y = MAX2(trans_y,
					   g_globals.dataset->dim_min[dim] +
					   g_globals.brushes[0].Size(dim) * .5);

		double ly	= trans_y - g_globals.brushes[0].Size(dim) * .5;
		double hy	= trans_y + g_globals.brushes[0].Size(dim) * .5;
		g_globals.brushes[0].SetMinMax(dim, ly, hy);
		m_isBrushModified = true;
		appBrushBoundaryCaption(canvas, dim);
	} else if (markerStatus == MST_DRAG_MIN)
	{
		double hy	= g_globals.brushes[0].Max(dim);
		double ly	= MIN2(move_y, hy);
		g_globals.brushes[0].SetMinMax(dim, ly, hy);
		m_isBrushModified = true;
		appBrushBoundaryCaption(canvas, dim);
	} else if (markerStatus == MST_DRAG_MAX)
	{
		double ly	= g_globals.brushes[0].Min(dim);
		double hy	= MAX2(move_y, ly);
		g_globals.brushes[0].SetMinMax(dim, ly, hy);
		m_isBrushModified = true;
		appBrushBoundaryCaption(canvas, dim);
	}

	if (markerStatus != MST_DRAG_MIN && markerStatus != MST_DRAG_MAX)
	{
		double brushMax, brushMin;
		GetNormalizedBrushExtents(&g_globals.brushes[0], dim, brushMax, brushMin);

		// check if mouse is on markers
		if (p.X >= 1.02f && p.X <= 1.03f)
		{
			double lo	= YMAP(brushMin);
			double hi	= YMAP(brushMax);
			if ((hi - p.Y) > (p.Y - lo))
				markerStatus = MST_ON_MAX;
			else
				markerStatus = MST_ON_MIN;
		} else
		{
			if (markerStatus == MST_ON_MIN || markerStatus == MST_ON_MAX)
				markerStatus = MST_BLANK;
		}
	}
	appCanvasBrushHasChanged(Togl_Interp(canvas->GetWidget()));
	appRefreshAllCanvas(Togl_Interp(canvas->GetWidget()));
}
//-----------------------------------------------------------------------------

void CFlatPixel::InputPixelMultiSelect(grCanvas *canvas, int sx, int sy)
{
	if (!multi_select)
		return;

	Vec2 p	= canvas->MapScreenToData(Vec2(sx, sy));
	int idx	= -1;
	m_subwindows[m_currDim]->Contains((float) p.X, (float) p.Y, idx);

	if (idx >= 0 && idx < g_globals.dataset->data_size) {
		multiSelectAddPoint(flat_query.indices[idx]);
//		m_isBrushModified = true;
	}
}

//-----------------------------------------------------------------------------
void CFlatPixel::InputButtonPress(int button, int x, int y)
{
	if (!g_globals.dataset)
		return;

	if (button == 2)
	{
		canvas->SetHandCursor();
		isPanning = true;
		return;
	}

	if (m_currDim != 0 && m_isShowingColormap)
	{
		int dim		= m_currDim - 1;
		Vec2 cursor	= MapScreenToCMapData(Vec2(x, y));
		double brushMax, brushMin;

		GetNormalizedBrushExtents(&g_globals.brushes[0], dim, brushMax, brushMin);
		if (cursor.X >= 0.98f && cursor.X <= 1.02f)
		{
			if (cursor.Y <= YMAP(brushMin) && cursor.Y >= YMAP(brushMax))
			{
				if (button == 0)
					markerStatus = MST_TRANSLATE;
				else if (button == 1)
				{
					markerStatus = MST_RESIZE;
					resize_y = YINVMAP(dim, cursor.Y);
				}
			}
		}
	}

	Vec2 p		= canvas->MapScreenToData(Vec2(x, y));
	int nearDim	= FindNearestDim(p);
	//Zaixian Start
	char cmdStr[255];
	//Zaixian End

	if (button == 0)
	{
		if (markerStatus == MST_ON_MIN)
			markerStatus = MST_DRAG_MIN;
		else if (markerStatus == MST_ON_MAX)
			markerStatus = MST_DRAG_MAX;
		else
		{
			if (nearDim != -1)
			{
				m_currDim = nearDim;
				//Zaixian Start
				if (m_currDim==0)
					strcpy(cmdStr, "::.flatPixelSettings::SetSelectedDim Combined_Distance");
				else
					sprintf(cmdStr, "::.flatPixelSettings::SetSelectedDim %s", 
						g_globals.dataset->all_names[m_currDim-1]);

				Tcl_Eval(Togl_Interp(canvas->GetWidget()), cmdStr);
				//Zaixian End
				//if (settingsWindow && settingsWindow->isVisible())
				//	settingsWindow->SetSelectedDim(m_currDim);
				if (canvas->shiftkey)
				{
					mode = MULTI_SELECT;
					InputPixelMultiSelect(canvas, x, y);
					return;
				}
				appRefreshAllCanvas(Togl_Interp(canvas->GetWidget()));
				//canvas->Refresh(true);
			}
		}
	} else if (button == 1)
	{
		if (m_currDim == nearDim)
		{
			if (m_sortedDim == nearDim)
			{
				m_sortAsc = !m_sortAsc;
				//Zaixian Start
				sprintf(cmdStr, "::.flatPixelSettings::SetSortingOrder %d", m_sortAsc);
				Tcl_Eval(Togl_Interp(canvas->GetWidget()), cmdStr);
				//Zaixian End
				//if (settingsWindow && settingsWindow->isVisible())
				//	settingsWindow->SetSortingOrder(m_sortAsc);
			} else
			{
				m_sortedDim = nearDim;
				//Zaixian Start
				if (m_sortedDim==0)
					strcpy(cmdStr, "::.flatPixelSettings::SetSortedByDim Combined_Distance"); 
				else
					sprintf(cmdStr, "::.flatPixelSettings::SetSortedByDim %s", 
						g_globals.dataset->all_names[m_sortedDim-1]);
				Tcl_Eval(Togl_Interp(canvas->GetWidget()), cmdStr);
				//Zaixian End
				//if (settingsWindow && settingsWindow->isVisible())
				//	settingsWindow->SetSortedByDim(m_sortedDim);
			}
			m_isBrushModified = true;
			canvas->Refresh();
		}
	}
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
int CFlatPixel::FindNearestDim(Vec2 p)
{
	if (!g_globals.dataset)
		return -1;
	for (int i = m_showingDerived; i < g_globals.dataset->dims + 1; ++i)
	{
		if (m_display == CFlatPixel::PDT_SPIRAL)
		{
			if (m_subwindows[i]->Contains((float) p.X, (float) p.Y))
				return i;
		}
	}
	return -1;
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CFlatPixel::InputButtonRelease(int button, int x, int y)
{
	if (button == 2)
		isPanning = false;

	if (mode != NONE) {
		mode = NONE;
		canvas->Refresh();
	}

	if (markerStatus == MST_DRAG_MIN ||
		markerStatus == MST_DRAG_MAX ||
		markerStatus == MST_TRANSLATE ||
		markerStatus == MST_RESIZE)
	{
		markerStatus = MST_BLANK;
		appRefreshAllCanvas(Togl_Interp(canvas->GetWidget()));
	}

	canvas->SetDefaultCursor();
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
static inline double norm(int dim, double val)
{
	return (val - g_globals.dataset->dim_min[dim]) / 
		(g_globals.dataset->dim_max[dim] - g_globals.dataset->dim_min[dim]);
}

void CFlatPixel::ComputeQueryDist(int sortedDim /*=0*/,bool isAscending /*=true*/)
{
	if (!g_globals.dataset)
		return;

	ARR<double> zero_combined_distances(g_globals.dataset->data_size);
	ARR<double> combined_distances(g_globals.dataset->data_size);

	flat_query.distances.SetLen((g_globals.dataset->all_dims+1)*
								 g_globals.dataset->data_size);
	flat_query.randomNums.SetLen(g_globals.dataset->data_size);
	flat_query.sizes.SetLen(g_globals.dataset->data_size);

	int count = 0;
	Brush &brush = g_globals.brushes[0];
	flat_query.data_size = g_globals.dataset->data_size;

	// for each data point
	for (int i=0; i<g_globals.dataset->data_size; ++i) 
	{
		combined_distances[i] = 0;
		zero_combined_distances[i] = 0;

		flat_query.indices = i;
		flat_query.randomNums[i] = g_globals.GetRandomNum();
		flat_query.sizes[i] = 1;

		// for each dimension
		for (int j=0; j<g_globals.dataset->all_dims; ++j) 
		{
			// get the proper index of the data item that we want
			int idx = i * (g_globals.dataset->all_dims+1) + (j+1);

			double val = g_globals.dataset->data_buf[i*g_globals.dataset->all_dims+j];
			// distance from the brush center to the data
			flat_query.distances[idx] = val - brush.Pos(j);
			if (val < brush.Min(j) || val > brush.Max(j)) {
				zero_combined_distances[i] += g_globals.dataset->weight[j] * flat_query.distances[idx];
			}

			// add all the normalized distances for this data point
			combined_distances[i] += g_globals.dataset->weight[j] * (norm(j, val) - norm(j, brush.Pos(j)))
				/*flat_query.distances[idx]*/;
		}
		// relevance factor is the inverse of the combined distance
		double val = flat_query.distances[i*(g_globals.dataset->all_dims+1)] = /*totalWeight - */fabs(combined_distances[i]);
		double zero_val = fabs(zero_combined_distances[i]);

		if (zero_val == 0)
			++count;

		// get the min and max relevance factors for normalization later on
		if (i == 0) {
			flat_query.dist_min = flat_query.dist_max = val;
		} else {
			if (val < flat_query.dist_min)
				flat_query.dist_min = val;
			if (val > flat_query.dist_max)
				flat_query.dist_max = val;
		}
	}

	//	int numDims = g_globals.dataset->dims;
	//	CDBSimilarityCalculator *cal = new CDBSimilarityCalculator(numDims, numBins);
	//
	//	double **cov_matrix = new double* [numDims];
	//	double **dim_pos = new double* [2];
	//	for (i = 0; i < numDims; i++) 
	//		cov_matrix[i] = new double [numDims];
	//	for (i = 0; i < 2; i++) 
	//		dim_pos[i] = new double [numDims];
	//
	//	cal->CalculateSimilarityMatrix(1);
	//	cal->GetSimilarityMatrix(cov_matrix);
	//	int sort_dim = cal->GetSortDimension();
	//	delete cal;

	flat_query.brush_level = 
		((flat_query.dist_max - flat_query.dist_min) / (g_globals.dataset->data_size-1)) * count;

	// sort the distances
	g_globals.dataset->SortDist(&flat_query, sortedDim, isAscending);
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
bool CFlatPixel::ShowCaption(grCanvas *canvas, int x, int y)
{
	Vec2 p	= canvas->MapScreenToData(Vec2(x, y));

	int n = -1, dim = -1;
	ostringstream oss;
	for (int i = 0; i < m_subwindows.Len(); ++i)
	{
		if (m_display == CFlatPixel::PDT_SPIRAL)
		{
			if (m_subwindows[i]->Contains((float) p.X, (float) p.Y, n))
				dim = i;
		}
	}

	if (dim >= 0)
	{
		for (int i = 0; i < g_globals.dataset->dims + 1; ++i)
		{
			if (dim == i)
				oss << "{";
			if (i == 0)
				oss << "Combined Distance";
			else
			{
				oss << g_globals.dataset->names[i - 1];
				if (n >= 0 && n < g_globals.dataset->data_size) {
					oss << "=" << g_globals.dataset->data_buf
						[ g_globals.dataset->all_dims * flat_query.indices[n] + i-1 ];
				}
			}
			if (dim == i)
				oss << "}";
			if (i != g_globals.dataset->dims)
				oss << ", ";
		}
		appShowCanvasMessage(Togl_Interp(canvas->GetWidget()),
							 oss.str().c_str(), canvas->name);
		return true;
	}

	return false;
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Draw coordinate axes name and min max.
void CFlatPixel::DrawAxesNameMinMax(grCanvas *canvas)
{
	glShadeModel(GL_SMOOTH);
	canvas->SetForeground(interface_color[TEXT]);

	Vec2 font_size	= canvas->GetDataFontSize("W");

	double max_str_size, x = 0.985f;

	if (m_currDim == 0)
	{
		max_str_size = g_globals.screen_distortion->paracoord_interval[all_dims - 1]; 
		canvas->DrawConfinedString(x, font_size.Y, "Combined Distance",
								   max_str_size);
	} else
	{
		max_str_size = g_globals.screen_distortion->paracoord_interval[m_currDim - 1] -
					   font_size.X * 5; 
		canvas->DrawConfinedString(x, font_size.Y,
								   g_globals.dataset->names[m_currDim - 1],
								   max_str_size);
		// Draw the MAX2 values of the dimension
		canvas->DrawConfinedString(x, -.51 * font_size.Y,
								   g_globals.dataset->GetMax(m_currDim - 1),
								   max_str_size);
		// Draw the MIN2 values of the dimension
		canvas->DrawConfinedString(x, 1.01f - font_size.Y,
								   g_globals.dataset->GetMin(m_currDim - 1),
								   max_str_size);
	}
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CFlatPixel::CalculateSegments()
{
	static const double DEG2RAD	= 0.017453292519943295;

	float fAlpha				= 360.0f / (g_globals.dataset->dims+1);
	m_segments.SetLen(g_globals.dataset->dims+1);

	// left line if first segment at 90 deg
	double dAngle				= 90.0;
	double dLeftAngleDeg		= dAngle;
	if (dLeftAngleDeg <= 0)
		dLeftAngleDeg = 360.0 + dLeftAngleDeg;
	double dLeftAngleRad= DEG2RAD *dAngle;
	double dxLeftLine	= cos(dLeftAngleRad);
	double dyLeftLine	= sin(dLeftAngleRad);
	double mLeftLine	= dyLeftLine / dxLeftLine;
	bool bLeftLineDepX	= true;
	if (abs(mLeftLine) > 1.0)
		bLeftLineDepX = false;

	for (int n = 0; n < g_globals.dataset->dims+1; n++)
	{
		// calculate the draw line, which is perpendicular to
		//  the segment-halving line
		dAngle -= fAlpha / 2.0f;
		double dDrawLineAngleDeg= dAngle + 90.0;
		if (dDrawLineAngleDeg < 0)
			dDrawLineAngleDeg = 360.0 + dDrawLineAngleDeg;
		double dDrawLineAngleRad= DEG2RAD *dDrawLineAngleDeg;
		double dDrawAngleX		= cos(dDrawLineAngleRad);
		double dDrawAngleY		= sin(dDrawLineAngleRad);
		double mDrawLine		= dDrawAngleY / dDrawAngleX;
		bool bDrawLineDepX		= true;
		if (abs(mDrawLine) > 1.0)
		{
			mDrawLine = dDrawAngleX / dDrawAngleY;
			bDrawLineDepX = false;
		}

		// calculate the right line
		dAngle -= fAlpha / 2.0f;
		double dRightAngleDeg	= dAngle;
		if (n + 1 == g_globals.dataset->dims)
			dRightAngleDeg = 90.0;
		if (dRightAngleDeg < 0)
			dRightAngleDeg = 360.0 + dRightAngleDeg;
		double dRightAngleRad	= DEG2RAD *dRightAngleDeg;
		double dxRightLine		= cos(dRightAngleRad);
		double dyRightLine		= sin(dRightAngleRad);
		double mRightLine		= dyRightLine / dxRightLine;
		bool bRightLineDepX		= true;
		if (abs(mRightLine) > 1.0)
			bRightLineDepX = false;

		int nMirrorSegY	= -1;
		int nMirrorSegX	= -1;
		int nMirrorSegXY= -1;
		bool bMirrored	= false;
		if (dRightAngleDeg < 90.0 && dRightAngleDeg >= 0)
		{
			nMirrorSegY = g_globals.dataset->dims - n - 1;
			if (g_globals.dataset->dims % 2 == 0)
			{
				nMirrorSegX = g_globals.dataset->dims / 2 - n - 1;
				nMirrorSegXY = g_globals.dataset->dims / 2 + n;
			}
		} else if (dLeftAngleDeg > 0 &&
				   dLeftAngleDeg < 90.0 &&
				   dRightAngleDeg <360.0 &&
				   dRightAngleDeg> 270.0)
			nMirrorSegY = g_globals.dataset->dims - n - 1;
		else if (dLeftAngleDeg <= 270.0 ||
				 dLeftAngleDeg >= 360.0 ||
				 dRightAngleDeg >= 270.0 ||
				 dRightAngleDeg <= 180.0 ||
				 g_globals.dataset->dims % 2 == 0)
			if (dLeftAngleDeg > 270.0 &&
				dLeftAngleDeg <360.0 &&
				dRightAngleDeg> 270.0 &&
				dRightAngleDeg < 360.0 &&
				g_globals.dataset->dims % 2 != 0)
				nMirrorSegY = g_globals.dataset->dims - n - 1;
			else
				bMirrored = true;
		if (3 == g_globals.dataset->dims)
		{
			nMirrorSegY = -1;
			nMirrorSegX = -1;
			nMirrorSegXY = -1;
			bMirrored = false;
		}

		// initialize the segment
//		m_segments[n] = (new Segment(g_globals.dataset->data_size, n, 
//							mLeftLine, dLeftAngleDeg, bLeftLineDepX, 
//							mRightLine, dRightAngleDeg, bRightLineDepX, 
//							mDrawLine, dDrawLineAngleDeg, bDrawLineDepX, 
//							bMirrored, nMirrorSegY, nMirrorSegX, 
//							nMirrorSegXY, m_segments));

		// right line of this segment is left line of next segment
		mLeftLine = mRightLine;
		dLeftAngleDeg = dRightAngleDeg;
		bLeftLineDepX = bRightLineDepX;
		if (0.0 == dLeftAngleDeg)
			dLeftAngleDeg = 360.0;
	}
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CFlatPixel::FreeSpace()
{
	m_sortedDim = 0; m_sortAsc = true;
	m_currDim = 0;
	//	zoomDim = false;
	for (int i = 0; i < m_images.Len(); ++i)
	{
		delete (Image *) m_images[i];
		delete (Subwindow *) m_subwindows[i];
		m_images[i] = NULL; m_subwindows[i] = NULL;
	}
	m_images.Clear(); m_subwindows.Clear();
	m_distortions.Clear();
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CFlatPixel::SetSortedDim(int dim)
{
	if (m_sortedDim == dim)
		return;
	m_sortedDim = dim;
	m_isBrushModified = true;
	canvas->Refresh();
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CFlatPixel::SelectDim(int dim)
{
	if (m_currDim == dim)
		return; 

	m_currDim = dim;
	canvas->Refresh();
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CFlatPixel::SetDimDistortion(int dim, float dist)
{
	assert(dist >= 0 && dist <= 1.0f);
	if (m_distortions[dim] == dist)
		return;

	hasDistortionChanged = true;
	m_distortions[dim] = dist;
	canvas->Refresh();
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CFlatPixel::SetSortDimOrder(bool isAscending)
{
	if (m_sortAsc == isAscending)
		return;

	m_sortAsc = isAscending;
	m_isBrushModified = true;
	canvas->Refresh();
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CFlatPixel::SetPlacementType(CFlatPixel::PlacementType pt)
{
	if (m_placementType == pt)
		return;

	m_placementType = pt;
	switch (pt)
	{
	case PPT_NORMAL:
		if (m_dispPos)
		{
			delete[] m_dispPos[0];
			delete[] m_dispPos[1];
			delete[] m_dispPos;
			m_dispPos = NULL;
		}
		break;
	case PPT_MDS:
		if (!m_dispPos)
		{
			m_dispPos = new double * [2];
			m_dispPos[0] = new double [g_globals.dataset->dims + 1];
			m_dispPos[1] = new double [g_globals.dataset->dims + 1];
		}
		break;
	default:
		assert(true);
	}

	m_isBrushModified = true;
	hasPlacementChanged = true;
	canvas->Refresh();
}
//-----------------------------------------------------------------------------

void CFlatPixel::SetSamplingFactor(float val)
{
	assert(val >= 0 && val <= 1.0f);
	m_sampleFactor = val;
	m_sampleStart = g_globals.GetRandomNum() * (1-m_sampleFactor);
	m_sampleEnd = m_sampleStart + m_sampleFactor;
	reset = true;
	canvas->Refresh();
}

double dimnorm(double num, const ARR<double> &data, int data_num, int dims,
			int dimension)
{
	double scale;
	int idx	= dimension;
	double max = num, min = num;

	////////////////////////////////////////get the max and min value of one dimension///////////////////////////////////////////
	for (int count = 0; count < data_num; ++count, idx += dims)
	{
		if (data[idx] > max)
			max = data[idx];
		if (data[idx] < min)
			min = data[idx];
	}

	if (max == min)
		scale = 1;
	else
		scale = (num - min) / (max - min);

	return scale;
}

static inline double simi(const ARR<double> &data, int data_num, int dims,
						  int dim_1, int dim_2)
{
	double scale_1, scale_2, total = 0;
	int count	= 0;
	int idx1 = dim_1, idx2 = dim_2;

	//////////////////////calculate distance between dimension 1 and dimension 2//////////////////////////////////////
	for (count = 0; count < data_num; ++count, idx1 += dims, 
		 		 												idx2 += dims)
	{
		scale_1 = dimnorm(data[idx1], data, data_num, dims, dim_1);
		scale_2 = dimnorm(data[idx2], data, data_num, dims, dim_2);

		double diff	= scale_1 - scale_2;
		total += diff * diff;
	}

	return sqrt(total);
}

void CFlatPixel::CalcPos(int n, int m, const ARR<double> &data)
{
	int row, column;
	double **simi_matrix= new double *[m]; // Similarity matrix of the n data points

	int count			= 0;
	for (count = 0; count < m; ++count)
		simi_matrix[count] = new double[m];

	/////////////////get the similarity matrix of the data set///////////////////////////
	for (row = 0 ; row < m; ++row)
		for (column = 0; column < m; ++column)
		{
			simi_matrix[row][column] = simi_matrix[column][row] = simi(data, n,
																	   m, row,
																	   column);
			//cout<<simi_matrix[row][column]<<endl;
		}

	multidimensional_scaling(m, 2, simi_matrix, m_dispPos);
}


