//modified by Jing Yang after 7/6/2001
#include <xmdv/ogl.h>
#include <xmdv/CHierDisplay.h>
#include <xmdv/XmdvTool.h>
#include <xmdv/colormap.h>
#include <xmdv/CFlatParcoord.h>
#include <xmdv/magnify.h>
#include <xmdv/brush_oper.h>
#include <xmdv/CDimReducer.h>
#include <xmdv/CDimReductionDisplayInformation.h>
#include <xmdv/CHierNav.h>

/* SID */
#include <backend/db_buffer.h>
#include <backend/db_clock.h>
#include <backend/db_prefetch.h>

#ifdef XMDV_USE_BACKEND
#include <cache/ICacheController.h>
#include <cache/ICacheEntry.h>
#endif XMDV_USE_BACKEND

CHierDisplay::CHierDisplay(CHierDisplayInformation* hier_info)
{
	this->hier_info= hier_info;
}


CHierDisplay::~CHierDisplay()
{
}


//set this->current_opoacity;
void CHierDisplay::SetCurrentOpacity(double opacity)
{
	current_opacity = opacity;
}


//draw the canvas;
void CHierDisplay::Draw()
{
	//IF the data base is active don't need to initialize the cluster tree.
	//this is an important step 
		if(!hier_info->database)
	{
		if (!g_globals.dataset) return;
		hierInitClusterTree();
		if ( !g_globals.btree ) return;
	}
	else
	{
		hierInitClusterTree();
		//This is an important statement it decides whether the Canvas is drawn or not.
		//Have to do this because the draw method in CHierNav is called each time there is 
		//any event. Therefore to filter this event we need this flag.
		g_globals.hier_info->databaseInfo.activeFlag = true;	
		//All the init functions that were previously called in hierInitClusterTree
		cfNavInit();
	}


	//set parameters used for drawing, to be rewritten in subclasses;
	SetParameters();
	//set canvas background, clear canvas, and draw axes, to be rewritten in subclasses;
	PrepareCanvas();

	canvas->bypassAll(true);

	glShadeModel(GL_SMOOTH);

	if ( ! hier_info->database )
	{
		//draw clusters in unbrushed region;
		DrawUnbrushedRegion();
	}
	//draw clusters in brushed region;
	DrawBrushedRegion();

	DrawAxes();
	canvas->bypassAll(false);
	canvas->Flush();

	//free space used for drawing, to be rewritten in subclasses;
	FreeSpace();

	if ( hier_info->database )
	{
		/* SID */
	}
}


//draw clusters in unbrushed region;
void CHierDisplay::DrawUnbrushedRegion()
{
	//if MAGNIGY, should not draw unbrushed region.
	if ( MAGNIFY ) 
		return;

	//if both means and bands of the clusters need not to be shown, return;
	if ( ! ( hier_info->SHOW_MEANS || hier_info->SHOW_BANDS ) )
		return;

	//set this->current_opacity;
	double opacity = 1.0;
	if (hier_info->fade_factor>0) 
		opacity=1-hier_info->fade_factor;
	SetCurrentOpacity( opacity );

	bool brushed = false;

	DrawClusters( brushed );


	/*	double left_limit, right_limit, level_of_detail;

	//draw the unbrushed region at left side of the brush; 
	left_limit = 0;
	right_limit = tree_brush[0];
	level_of_detail = cluster_radius;     //cluster_radius is the level of detail of the unbrushed region.

	DrawClusters( left_limit, right_limit, level_of_detail, brushed );

	//draw the unbrushed region at right side of the brush; 
	left_limit = tree_brush[1];
	right_limit = 1;

	DrawClusters( left_limit, right_limit, level_of_detail, brushed );
	*//*
	//if both means and bands of the clusters need not to be shown, return;
	if ( ! ( SHOW_MEANS || SHOW_BANDS ) )
	return;

	//set this->current_opacity;
	double opacity = 1.0;
	if (fade_factor>0) 
	opacity=1-hier_info->fade_factor;
	SetCurrentOpacity( opacity );

	double left_limit, right_limit, level_of_detail;
	bool brushed;

	//draw the unbrushed region at left side of the brush; 
	left_limit = 0;
	right_limit = hier_info->tree_brush[0];
	level_of_detail = hier_info->cluster_radius;     //cluster_radius is the level of detail of the unbrushed region.
	brushed = false;

	DrawClusters( left_limit, right_limit, level_of_detail, brushed );

	//draw the unbrushed region at right side of the brush; 
	left_limit = hier_info->tree_brush[1];
	right_limit = 1;

	DrawClusters( left_limit, right_limit, level_of_detail, brushed );
	*/
}


//draw clusters in unbrushed region;
void CHierDisplay::DrawBrushedRegion()
{
	/*	//if both means and bands of the clusters need not to be shown, return;
	if ( ! ( hier_info->SHOW_MEANS || hier_info->SHOW_BANDS ) )
	return;

	//set current opacity
	double opacity = 1.0;
	if (hier_info->fade_factor<0) 
	opacity = 1+hier_info->fade_factor;
	SetCurrentOpacity( opacity );

	bool brushed = true;
	DrawClusters( brushed );
	//////

	double left_limit, right_limit, level_of_detail;
	//draw the brushed region; 
	left_limit = tree_brush[0];
	right_limit = tree_brush[1];
	level_of_detail = brushed_radius;     //brushed_radius is the level of detail of the brushed region.
	brushed = true;

	DrawClusters(left_limit, right_limit, level_of_detail, brushed);*/


	//if both means and bands of the clusters need not to be shown, return;
	if ( ! ( hier_info->SHOW_MEANS || hier_info->SHOW_BANDS ) )
		return;

	//set current opacity
	double opacity = 1.0;
	if (hier_info->fade_factor<0) 
		opacity = 1+hier_info->fade_factor;
	SetCurrentOpacity( opacity );

	double left_limit, right_limit, level_of_detail;
	bool brushed;

	//draw the brushed region; 
	left_limit = hier_info->tree_brush[0];
	right_limit = hier_info->tree_brush[1];
	level_of_detail = hier_info->brushed_radius;     //brushed_radius is the level of detail of the brushed region.
	brushed = true;

	DrawClusters(left_limit, right_limit, level_of_detail, brushed);

}


void CHierDisplay::TraceDrawBrushedClusters( const Cluster &node )
{
	if ( ! BITTEST(node.flags,IS_BRUSHED_ANY) )
		return;

	if ( BITTEST(node.flags,IS_CONTOUR1) )
	{
		if ( BITTEST(node.flags,IS_BRUSHED) )
			DrawSingleCluster1(node, true);
		else 
			return;
	}
	else
	{
		for (int j=0; j<node.num_children; j++) 
			TraceDrawBrushedClusters(g_globals.btree->nodes[node.child[j]]);
	}
}


void CHierDisplay::TraceDrawUnbrushedClusters( const Cluster &node )
{
	if ( BrushSemantic == 0 )
	{
		if ( BITTEST(node.flags,IS_BRUSHED) )
			return;
	}

	if ( BITTEST(node.flags,IS_CONTOUR0) )
	{
		if ( ! BITTEST(node.flags,IS_BRUSHED) )
			DrawSingleCluster1(node, false);
		else 
			return;
	}
	else
	{
		for (int j=0; j<node.num_children; j++) 
			TraceDrawUnbrushedClusters(g_globals.btree->nodes[node.child[j]]);
	}
}

//Given left limit, right limit and level of detail, draw clusters inside this region.
void CHierDisplay::DrawClusters( double left_limit, double right_limit, double level_of_detail, 
								bool brushed )
{
	if ( hier_info->database )
	{
		#ifdef XMDV_USE_BACKEND
		/* SID */
		Cluster c;
		c.sx.SetLen(g_globals.dataset->all_dims);
		c.mean.SetLen(g_globals.dataset->all_dims);
		c.max.SetLen(g_globals.dataset->all_dims);
		c.min.SetLen(g_globals.dataset->all_dims);
		c.all_mean.SetLen(g_globals.dataset->all_dims);
		c.num_children = 0;
		c.entries = 0;
		
		cache::ICacheController* cacheController = g_globals.hier_info->databaseInfo.cacheBuilder->getCacheController();
		Iterator<cache::ICacheEntry*>* dataIterator = 
			cacheController->load(left_limit,right_limit,level_of_detail,0);


		using namespace std;

		cache::ICacheEntry* cacheEntry;

		while(!dataIterator->isDone())
		{			
			cacheEntry = dataIterator->get_next();				

			c.color = (cacheEntry->x + cacheEntry->y)/2;

			c.sqrt_radius = cacheEntry->l;

			for (unsigned int i=0; i < cacheEntry->data->numberOfDimensions; i++)
			{

				// Compute the mean
				if (cacheEntry->n > 0) {

					double sum_dim = cacheEntry->data->data[i] * 
						(g_globals.dataset->cg_dim_max[i]-g_globals.dataset->cg_dim_min[i]) +
						(cacheEntry->n * g_globals.dataset->cg_dim_min[i]);

					c.mean[i] = sum_dim/cacheEntry->n;
				}
				else
				{				
					c.mean[i] = cacheEntry->data->data[i];
				}

				/*Not Sclaing the value any more * 
				(g_globals.dataset->all_dim_max[i] - g_globals.dataset->all_dim_min[i]) 
				+ g_globals.dataset->all_dim_min[i];*/

				c.max[i] = c.mean[i] + c.sqrt_radius;

				c.min[i] = c.mean[i] - c.sqrt_radius;

				if ( c.min[i] < g_globals.dataset->all_dim_min[i] )
				{
					c.min[i] = g_globals.dataset->all_dim_min[i];
				}
				if ( c.max[i] > g_globals.dataset->all_dim_max[i] )
				{
					c.max[i] = g_globals.dataset->all_dim_max[i];
				}
			}

			DrawSingleCluster1(c, brushed);
		}    
		//Deleting the data iterator
		delete(dataIterator);
		#endif XMDV_USE_BACKEND
	}
	else 
	{
		if ( brushed )
		{
			TraceDrawBrushedClusters( g_globals.btree->nodes[0] );
		}
		else
		{
			TraceDrawUnbrushedClusters( g_globals.btree->nodes[0] );
		}

	}  
}


void CHierDisplay::DrawClusters( bool brushed )
{
	if ( brushed )
	{
		TraceDrawBrushedClusters( g_globals.btree->nodes[0] );
	}
	else
	{
		TraceDrawUnbrushedClusters( g_globals.btree->nodes[0] );
	}
}
//jing 
//draw a single cluster
void CHierDisplay::DrawSingleCluster1(const Cluster &c, bool brushed)
{
	Cluster mappedNode;
	mappedNode.mean.SetLen(g_globals.dataset->dims);
	mappedNode.max.SetLen(g_globals.dataset->dims);
	mappedNode.min.SetLen(g_globals.dataset->dims);
	mappedNode.sx.SetLen(g_globals.dataset->dims);

	mappedNode.num_children = c.num_children;
	mappedNode.color = c.color;
	mappedNode.flags = c.flags;
	mappedNode.entries = c.entries;

	if ( !show_two_axes )
	{
		g_globals.dataset->MapData(c.mean, mappedNode.mean);
		g_globals.dataset->MapData(c.max, mappedNode.max);
		g_globals.dataset->MapData(c.min, mappedNode.min);
		g_globals.dataset->MapData(c.sx, mappedNode.sx);
		DrawSingleCluster(mappedNode, brushed);
	}
	else
	{
		ARR<double> mean_max;
		ARR<double> mean_min;
		ARR<double> max_max;
		ARR<double> max_min;
		ARR<double> min_max;
		ARR<double> min_min;

		mean_max.SetLen(g_globals.dataset->dims);
		mean_min.SetLen(g_globals.dataset->dims);
		max_max.SetLen(g_globals.dataset->dims);
		max_min.SetLen(g_globals.dataset->dims);
		min_max.SetLen(g_globals.dataset->dims);
		min_min.SetLen(g_globals.dataset->dims);

		g_globals.dataset->MapDataWithDimClusterExtends(c.mean, mappedNode.mean, mean_max, mean_min);
		g_globals.dataset->MapDataWithDimClusterExtends(c.max, mappedNode.max, max_max, max_min);
		g_globals.dataset->MapDataWithDimClusterExtends(c.min, mappedNode.min, min_max, min_min);
		g_globals.dataset->MapData(c.sx, mappedNode.sx);
		DrawSingleClusterWithTwoAxes(mappedNode, mean_max, mean_min, max_max, max_min, min_max, min_min, brushed);
	}
}



/******************** the following functions need to be rewritten in the subclasses *****************/
/*****************************************************************************************************/


//draw a single cluster, to be rewritten by subclasses;
void CHierDisplay::DrawSingleCluster(const Cluster &c, bool brushed)
{
}


//draw a single cluster, to be rewritten by subclasses;
void CHierDisplay::DrawSingleClusterWithTwoAxes(const Cluster &c, ARR<double> mean_max, ARR<double> mean_min, ARR<double> max_max, 
												ARR<double> max_min, ARR<double> min_max, ARR<double> min_min, bool brushed)
{
}


//set parameters used for the drawing; to be rewritten by subclasses;
void CHierDisplay::SetParameters()
{
	dims = g_globals.dataset->dims;

	if ( g_globals.dim_reducer != NULL && g_globals.dim_reducer->status == APPLY && g_globals.dim_reduction_info->disagree_present == MEANBANDS )
		show_dimcluster_band = true;
	else
		show_dimcluster_band = false;

	if ( g_globals.dim_reducer != NULL && g_globals.dim_reducer->status == APPLY && g_globals.dim_reduction_info->disagree_present == TWOAXES )
		show_two_axes = true;
	else
		show_two_axes = false;

	if ( g_globals.dim_reducer != NULL && g_globals.dim_reducer->status == APPLY && g_globals.dim_reduction_info->disagree_present == WIDEAXES )
		show_wide_axes = true;
	else
		show_wide_axes = false;
}


//free space used for drawing; to be rewritten by subclasses;
void CHierDisplay::FreeSpace()
{
}


void CHierDisplay::DrawAxes()
{
}


/******************** the above functions need to be rewritten in the subclasses *********************/
/*****************************************************************************************************/
