#include <xmdv/CHierParcoord.h>
#include <xmdv/ogl.h>
#include <xmdv/cluster.h>
#include <xmdv/XmdvTool.h>
#include <xmdv/colormap.h>
#include <xmdv/magnify.h>
#include <xmdv/brush_oper.h>
#include <xmdv/CDimReducer.h>
#include <xmdv/CScreenDistortion.h>

CHierParcoord::CHierParcoord(CHierDisplayInformation* hier_info) : CHierDisplay(hier_info)
{
	axis = -1;
	xor_needs_undo = false;

	mapedX = NULL;
	mapedYMean = NULL;
	mapedYMax = NULL;
	mapedYMin = NULL;
	para = NULL;
}

CHierParcoord::~CHierParcoord()
{
	FreeSpace();
}

void CHierParcoord::Draw() 
{
	CHierDisplay::Draw();
}

/*
void CHierParcoord::DrawMeansAndBands( double left_limit, double right_limit, double level_of_detail, 
									   bool brushed )
{
	prepare( left_limit, right_limit, level_of_detail, NO_P_ENTRIES-1);
	
	ENTRY *entry;
	if ( brushed )
	{
        canvas->SetForeground(interface_color[SBB], current_opacity);
        glLineWidth(1.5);
    } 
	else 
	{
        glLineWidth(1.0);
    }

	for (init(); (entry=get_one())!=NULL; )
	{
		c.color = (entry->data->X+entry->data->Y)/2.;
		c.sqrt_radius = entry->data->L;
		for (int i=0; i<TABLE_ND; i++)
			c.mean[i] = entry->data->D[i];

		DrawSingleNodeMeanAndBand(c, 0.5, brushed);
	}

    glLineWidth(1.0);
}
*/

//draw the mean and band of one cluster
void CHierParcoord::DrawSingleCluster(const Cluster &c, bool brushed)
{
	//map the mean of a cluster to a polygonal line
	int d;
    for (d=0; d<dims; d++)
    {
        poly[d].X = mapedX[d];
        poly[d].Y = mapedYMean[d] = YMAP(d,c.mean[d]);
    }

	//draw band
    if ( hier_info->SHOW_BANDS )
	{
		for (d=0; d<dims; d++)
		{
			mapedYMax[d] = YMAP(d,c.max[d]);
			mapedYMin[d] = YMAP(d,c.min[d]);
			if ( fabs(1 - hier_info->extent_scaling_fac ) > 1e-4 )
			{
				mapedYMax[d] = mapedYMean[d] + hier_info->extent_scaling_fac * (mapedYMax[d] - mapedYMean[d]);
				mapedYMin[d] = mapedYMean[d] + hier_info->extent_scaling_fac * (mapedYMin[d] - mapedYMean[d]);
			}
		}

		DrawSingleNodeBand(c, hier_info->density, brushed);
	}
    
	//draw mean
    if ( hier_info->SHOW_MEANS )
    {
        if ( brushed )
	    {
			canvas->bypass(1);
            canvas->SetForeground(interface_color[SBB], current_opacity);
            glLineWidth(1.5);
        } 
	    else 
	    {
            int color_idx = int(c.color*(CMAPSIZE-1));
            canvas->SetForeground( (*hier_info->CMAP)[color_idx], current_opacity);
            glLineWidth(1.0);
        }

		canvas->DrawLines(poly);
    }
}

//draw a band for one cluster
void CHierParcoord::DrawSingleNodeBand(const Cluster &c, double density, bool brushed)
{
/*    //if the cluster has only one children, there is no band.
    if ( c.num_children == 0 ) 
        return;
*/
    //Set color
    if (hier_info->COLORED_BANDS) 
    {

        int color_idx = (int)(c.color*(CMAPSIZE-1));
        canvas->SetForeground( (*hier_info->CMAP)[color_idx], current_opacity);
    }
    else
        canvas->SetForeground(interface_color[BRUSH1+2]);

    int d;
	ARR<Vec2> band(4);
    // Draw the lower band

    lo.X = mapedX[0];
    lo.Y = mapedYMin[0];

    hi.X = lo.X;
    hi.Y = mapedYMean[0];


    band[0] = lo;
    band[3] = hi;
    for (d=1; d<poly_length; d++)
    {
        lo.X = mapedX[d];
        lo.Y = mapedYMin[d];

        hi.X = lo.X;
        hi.Y = mapedYMean[d];

        band[1] = lo;
        band[2] = hi;
        canvas->DrawTransparentBand(band, 0.0, density, canvas->fg_color, canvas->fg_color);

        band[0] = band[1];
        band[3] = band[2];
    }

    // Draw the upper band
    lo.X = mapedX[0];
    lo.Y = mapedYMean[0];

    hi.X = lo.X;
    hi.Y = mapedYMax[0];
    band[0] = lo;
    band[3] = hi;

    for (d=1; d<poly_length; d++)
    {
        lo.X = mapedX[d];
        lo.Y = mapedYMean[d];

        hi.X = lo.X;
        hi.Y = mapedYMax[d];

		band[1] = lo;
        band[2] = hi;

        canvas->DrawTransparentBand(band, density, 0.0, canvas->fg_color, canvas->fg_color);

        band[0] = band[1];
        band[3] = band[2];
	}
}


void CHierParcoord::DrawSingleClusterWithTwoAxes(const Cluster &c, ARR<double> mean_max, ARR<double> mean_min, ARR<double> max_max, 
                                                ARR<double> max_min, ARR<double> min_max, ARR<double> min_min, bool brushed)
{
	//map the mean of a cluster to a polygonal line
	int j, d;
    for (d=0; d<poly_length; d++)
    {
        poly[d].X = mapedX[d];
    }

    d = 0; 
    for (j=0; j<dims; j++) 
    {
        if ( g_globals.dim_reducer->IsLeafDim(j) )
	    {
            poly[d].Y = mapedYMean[d] = YMAP(j,c.mean[j]);
            d++;
	    }
        else
        {
            poly[d].Y = mapedYMean[d] = YMAP(mean_min[j]);
            d++;

            poly[d].Y = mapedYMean[d] = YMAP(c.mean[j]);
            d++;
		    
            poly[d].Y = mapedYMean[d] = YMAP(mean_max[j]);
            d++;
        }
    }

	//draw band
    if ( hier_info->SHOW_BANDS )
	{
        d = 0; 
        for (j=0; j<dims; j++) 
        {
            if ( g_globals.dim_reducer->IsLeafDim(j) )
	        {
                mapedYMax[d] = YMAP(j,c.max[j]);
                mapedYMin[d] = YMAP(j,c.min[j]);
                d++;
	        }
            else
            {
                mapedYMax[d] = YMAP(max_min[j]);
                mapedYMin[d] = YMAP(min_min[j]);
                d++;
                mapedYMax[d] = YMAP(c.max[j]);
                mapedYMin[d] = YMAP(c.min[j]);
                d++;
                mapedYMax[d] = YMAP(max_max[j]);
                mapedYMin[d] = YMAP(min_max[j]);
                d++;
            }
        }
		for (d=0; d<poly_length; d++)
		{
			if ( fabs(1 - hier_info->extent_scaling_fac ) > 1e-4 )
			{
				mapedYMax[d] = mapedYMean[d] + hier_info->extent_scaling_fac * (mapedYMax[d] - mapedYMean[d]);
				mapedYMin[d] = mapedYMean[d] + hier_info->extent_scaling_fac * (mapedYMin[d] - mapedYMean[d]);
			}
		}

		DrawSingleNodeBand(c, hier_info->density, brushed);
	}
    
	//draw mean
    if ( hier_info->SHOW_MEANS )
    {
        if ( brushed )
	    {
            canvas->SetForeground(interface_color[SBB], current_opacity);
            glLineWidth(1.5);
        } 
	    else 
	    {
            int color_idx = int(c.color*(CMAPSIZE-1));
            canvas->SetForeground( (*hier_info->CMAP)[color_idx], current_opacity);
            glLineWidth(1.0);
        }

		canvas->DrawLines(poly);
    }
}


void CHierParcoord::SetParameters()
{
    CHierDisplay::SetParameters();
	SetCalcVals();

    int i, j, d;
    if ( !show_two_axes )
        poly_length = g_globals.dataset->dims;
    else
    {
        poly_length = 0;
        for ( i = 0; i < g_globals.dataset->dims; i++ )
        {
            if ( g_globals.dim_reducer->IsLeafDim(i) )
                poly_length ++;
            else 
                poly_length += 3;
        }
    } 
    
    poly.SetLen(poly_length);
    poly.SetLen(poly_length);
	mapedX = new double [poly_length];
	mapedYMean = new double [poly_length];
	mapedYMax = new double [poly_length];
	mapedYMin = new double [poly_length];
	para = new double [poly_length];

    if ( !show_two_axes )
    {
        for ( d = 0; d < dims; d++ )
	    {
		    mapedX[d] = XMAP(d);
		    para[d] = 1./(g_globals.dataset->dim_max[d] - g_globals.dataset->dim_min[d]);
	    }
    }
    else
    {
        d = 0; 
        for (j=0; j<dims; j++) 
        {
            if ( g_globals.dim_reducer->IsLeafDim(j) )
	        {
		        mapedX[d] = XMAP(j);
		        para[d] = 1./(g_globals.dataset->dim_max[j] - g_globals.dataset->dim_min[j]);
                d++;
	        }
            else
            {
		        mapedX[d] = XMINMAP(j);
		        para[d] = 1.;
                d++;

                mapedX[d] = XMAP(j);
		        para[d] = 1.;
                d++;
		        
                mapedX[d] = XMAXMAP(j);
		        para[d] = 1.;
                d++;
            }
        }
    }

}


void CHierParcoord::FreeSpace()
{
	if (mapedX) { delete [] mapedX; mapedX = NULL; }
	if (mapedYMean) { delete [] mapedYMean; mapedYMean = NULL; }
	if (mapedYMax) { delete [] mapedYMax; mapedYMax = NULL; }
	if (mapedYMin) { delete [] mapedYMin; mapedYMin = NULL; }
	if (para) { delete [] para; para = NULL; }
}


void CHierParcoord::DrawAxes()
{
    if ( show_dimcluster_band )
    {
        DrawParcoordAxesWithBands(canvas);
    }
    else if ( show_two_axes )
    {
        DrawParcoordAxesWithTwoAxes(canvas);
    }
    else if ( show_wide_axes )
    {
        DrawParcoordWideAxes(canvas);
    }
    else 
    {
        DrawParcoordAxes(canvas);
    }
	DrawNominalOnAxes(canvas);
}

 
void CHierParcoord::InputMouseMotion(int x, int y)
{
	if (!g_globals.dataset) return;

    if ( g_globals.screen_distortion->distortion_status )
    {
        Vec2 p = canvas->MapScreenToData( Vec2(x,y) );
        InputParacoordFindAxesBetween(canvas, p.X, p.Y, axis);
        return;
    }

    ParcoordCaption(canvas, x, y);
    static int prev_sx, prev_sy;

    switch (input_mode) {

        case CANVAS_PAN:
            appCanvasMousePan(canvas);
            return;

        default: break;
    }

    prev_sx = x;
    prev_sy = y;

}


void CHierParcoord::InputButtonPress(int button, int x, int y)
{
    if (!g_globals.dataset) return;

    input_mode = NONE;

    if (canvas->button[2] == true) {
        input_mode = CANVAS_PAN;
        canvas->SetHandCursor();
        return;
    }

    InputMouseMotion(x,y);
}


void CHierParcoord::InputButtonRelease(int button, int x, int y)
{
    if (!g_globals.dataset) return;

    if ( g_globals.screen_distortion->distortion_status )
    {
        double delta;

        input_mode = INCREASE_AXES_INTERVAL;
        switch (input_mode) 
        {
            case INCREASE_AXES_INTERVAL:
                delta = 1. / g_globals.dataset->dims * 0.2;
                InputParacoordScaleAxes(canvas, axis, delta);
                canvas->Refresh();
                break;
            case DECREASE_AXES_INTERVAL:
                delta = (-1.) / g_globals.dataset->dims * 0.2;
                InputParacoordScaleAxes(canvas, axis, delta);
                break;
            default: break;
        }
        return;
    }

     if (canvas->controlkey && button == 2 && g_globals.dim_reducer != NULL && g_globals.dim_reducer->status == APPLY)
    {
        Vec2 p = canvas->MapScreenToData( Vec2(x,y) );
        InputParacoordFindAxesNear(canvas, p.X, p.Y, axis);

        if (axis < 0 || axis >= g_globals.dataset->dims) 
        {
            axis = -1;
            return;
        }

        if ( ! canvas->altkey )
            g_globals.dim_reducer->ExtendNodeInRepresentativeDimList(axis);
        else
            g_globals.dim_reducer->ShinkNodeInRepresentativeDimList(axis);

        appRefreshAllCanvas(Togl_Interp(canvas->GetWidget()));

    }

     switch (input_mode) {

        case CANVAS_PAN:
            canvas->SetDefaultCursor();
            break;

        default: break;
    }

    input_mode = NONE;
}


bool CHierParcoord::TestDimensionality()  
{
	return ParcoordTestDimensionality();
}
