///////////////////////////////////////////////////////////////////////////////
/// \file CHierPixel.cpp
/// \brief Implementation of the CHierPixel class
/// \date 5/24/2004
/// \author Anilkumar Patro
///////////////////////////////////////////////////////////////////////////////

// SYSTEM INCLUDES
//
//#include <cstdlib>
//#include <cstring>
#include <sstream>
using namespace std;

// PROJECT INCLUDES
//
#include <xmdv/CHierPixel.h>
#include <xmdv/XmdvTool.h>
#include <xmdv/ogl.h>
#include <xmdv/Image.h>
#include <xmdv/Subwindow.h>
#include <xmdv/CScreenDistortion.h>
#include <xmdv/CHierNav.h>

// DEFINES
//
#define NUM_DERIVED_DIMS	1

// FORWARD REFERENCES
//

namespace
{

double resize_y;
bool isBrushModified;

};

//-----------------------------------------------------------------------------
CHierPixel::CHierPixel(CHierDisplayInformation *hier_info)
	: CHierDisplay(hier_info)
{
	m_currDim = 0;
	m_zoomDim = false;
	m_sortedDim = 0;
	m_isAscending = true;

	//Zaixian start
	m_isShowingColormap = true;
	m_showingDerived = 0;
	m_hasDistortionChanged = true;
	m_reset = true;
	//Zaixian end
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
CHierPixel::CHierPixel(const CHierPixel &from)
	: CHierDisplay(from.hier_info)
{
	// TODO: Add copy constructor
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
/*virtual*/ CHierPixel::~CHierPixel()
{
	for (int i = 0; i < m_images.Len(); ++i)
	{
		delete (Image *) m_images[i];
		delete (Subwindow *) m_subwindows[i];
		m_images[i] = NULL; m_subwindows[i] = NULL;
	}
	m_images.Clear();
	m_subwindows.Clear();
	m_distortions.Clear();
}
//-----------------------------------------------------------------------------

//-------------------------------------------------------------------------
CHierPixel & CHierPixel::operator =(const CHierPixel &from)
{
	if (this == &from)
		return *this;

	// TODO: Add assignment operator

	return *this;
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CHierPixel::PrepareCanvas()
{
	canvas_mode = DISP_2D;
	//	canvas_mode = DISP_3D;
	CDisplay::PrepareCanvas();
}
//-----------------------------------------------------------------------------

void CHierPixel::Draw()
{
	CHierDisplay::Draw();
//  editting code for backend begins - Charudatta
	//	//set canvas background, clear canvas, and draw axes, to be rewritten in subclasses;
	////	PrepareCanvas();
	////	canvas->DrawXZGrid();
	////	return;
	//
	//	//IF the data base is active don't need to initialize the cluster tree.
	//	if(!hier_info->database)
	//	{
	//		if (!g_globals.dataset) return;
	//		hierInitClusterTree();
	//		if ( !g_globals.btree ) return;
	//	}
	//	else
	//	{
	//		//This is an important statement it decides whether the Canvas is drawn or not.
	//		//Have to do this because the draw method in CHierNav is called each time there is 
	//		//any event. Therefore to filter this event we need this flag.
	//		g_globals.hier_info->databaseInfo.activeFlag = true;	
	//		//All the init functions that were previously called in hierInitClusterTree
	//		cfNavInit();
	//	}
	//
	//
	//	//set parameters used for drawing, to be rewritten in subclasses;
	//	SetParameters();
	//	//set canvas background, clear canvas, and draw axes, to be rewritten in subclasses;
	//	PrepareCanvas();
	//
	//	glShadeModel(GL_SMOOTH);
	//
	//	DrawBrushedRegion();
	//
	//	if ( ! hier_info->database )
	//	{
	//		//draw clusters in unbrushed region;
	//		DrawUnbrushedRegion();
	//	}
	//
	//	//draw clusters in brushed region;
	////	DrawAllClusters();
	//
	//	DrawAxes();
	//
	//	//free space used for drawing, to be rewritten in subclasses;
	//	FreeSpace();
	//editting ends
}
//-----------------------------------------------------------------------------
void CHierPixel::DrawAxes()
{
	ComputeQueryDist(m_sortedDim, m_isAscending);

	DrawSpiral();

	if (!m_isShowingColormap) {
		CHierDisplay::DrawAxes();
		return;
	}

	glPushMatrix();
	glLoadIdentity();

	// draw background
	glLineWidth(45.0f);
	RGBt color	(interface_color[BG]);
	glColor3f((float) color.R, (float) color.G, (float) color.B);
	glBegin(GL_LINE_STRIP); {
		glVertex2f(1.015f, -0.04f);
		glVertex2f(1.015f, 1.04f);
	} glEnd();

	// get brush extents
	if (m_currDim < NUM_DERIVED_DIMS)
	{
		//		g_globals.pixelUnbrushedCMap->DrawColorBar(1.0f, 1.0f, 1.0f, 0.0f, 20.0f);
		//	} else {
		//		double brushMax = hier_info->tree_brush[0], brushMin = hier_info->tree_brush[1];

		QueryDist &query_dist	= hier_query;
		double bb				= (query_dist.brush_level - query_dist.dist_min) /
								  (query_dist.dist_max - query_dist.dist_min);

		g_globals.pixelBrushedCMap->DrawColorBar(1.0f, 1.0f, 1.0f,
												 (float) YMAP(bb), 20.0f);
		g_globals.pixelUnbrushedCMap->DrawColorBar(1.0f, (float) YMAP(bb), 1.0f,
												   0.0f, 20.0f);

		// draw the colormap
		//		hier_info->CMAP.DrawColorBar(1.0f, 1.0f, 1.0f, 0, 20.0f);
		//		g_globals.pixelUnbrushedCMap->DrawColorBar(1.0f, 0.0f, 1.0f, 1.0f-brushMax, 20.0f);
		//		g_globals.pixelUnbrushedCMap->DrawColorBar(1.0f, 1.0f, 1.0f, 1.0f-brushMin, 20.0f);
		//	m_cmaps[m_currDim]->DrawColorBar(1.0f, 0.0f, 1.0f, 1.0f-brushMax, 20.0f);
		//	m_cmaps[m_currDim]->DrawColorBar(1.0f, 1.0f, 1.0f, 1.0f-brushMin, 20.0f);
		//		glLineWidth(1.0f);

		// draw markers
		//		glBegin(GL_TRIANGLES); {
		////			if (m_markerStatus == MST_ON_MIN || m_markerStatus == MST_DRAG_MIN)
		////				glColor3f(1.0f,0,0);
		////			else
		//				glColor3f(0,0,0);
		//			glVertex2f(1.02f, 1.0f-brushMin);
		//			glVertex2f(1.03f, 1.0f-brushMin+0.01f);
		//			glVertex2f(1.03f, 1.0f-brushMin-0.01f);
		//
		////			if (m_markerStatus == MST_ON_MAX || m_markerStatus == MST_DRAG_MAX)
		////				glColor3f(1.0f,0,0);
		////			else
		//				glColor3f(0,0,0);
		//			glVertex2f(1.02f, 1.0f-brushMax);
		//			glVertex2f(1.03f, 1.0f-brushMax+0.01f);
		//			glVertex2f(1.03f, 1.0f-brushMax-0.01f);
		//		} glEnd();
		// draw labels
		//		if (!m_isPanning)
		//			DrawAxesNameMinMax(canvas);
	} else
	{
		double brushMax, brushMin;
		GetNormalizedBrushExtents(&g_globals.brushes[0], m_currDim - 1,
								  brushMax, brushMin);
		double brushCenter	= ((brushMax - brushMin) * 0.5) + brushMin;

		// draw the colormap
		g_globals.pixelBrushedCMap->DrawColorBar(1.0f,
												 (float) YMAP(brushCenter),
												 1.0f, (float) YMAP(brushMax),
												 20.0f);
		g_globals.pixelBrushedCMap->DrawColorBar(1.0f,
												 (float) YMAP(brushCenter),
												 1.0f, (float) YMAP(brushMin),
												 20.0f);
		g_globals.pixelUnbrushedCMap->DrawColorBar(1.0f, (float) YMAP(brushMin),
												   1.0f, 1.0f, 20.0f);
		g_globals.pixelUnbrushedCMap->DrawColorBar(1.0f, (float) YMAP(brushMax),
												   1.0f, 0.0f, 20.0f);
		glLineWidth(1.0f);
		RGBt color	(interface_color[TEXT]);

		// draw markers
		glBegin(GL_TRIANGLES); {
			//			if (markerStatus == MST_ON_MIN || markerStatus == MST_DRAG_MIN)
			//				glColor3f(1.0f,0,0);
			//			else {
			glColor3f((float) color.R, (float) color.G, (float) color.B);
			//			}
			glVertex2f(1.02f, (float) YMAP(brushMin));
			glVertex2f(1.03f, (float) YMAP(brushMin) + 0.01f);
			glVertex2f(1.03f, (float) YMAP(brushMin) - 0.01f);

			//			if (markerStatus == MST_ON_MAX || markerStatus == MST_DRAG_MAX)
			//				glColor3f(1.0f,0,0);
			//			else {
			glColor3f((float) color.R, (float) color.G, (float) color.B);
			//			}
			glVertex2f(1.02f, (float) YMAP(brushMax));
			glVertex2f(1.03f, (float) YMAP(brushMax) + 0.01f);
			glVertex2f(1.03f, (float) YMAP(brushMax) - 0.01f);
		} glEnd();


		// draw labels
		//		if (!isPanning)
		DrawAxesNameMinMax(canvas);
	}

	glPopMatrix();

	// call base class implementation
	CHierDisplay::DrawAxes();
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CHierPixel::DrawSpiral()
{
	// figure out the size of each subwindow in terms of pixels
	static int size = 0;
	static int imgsize = 0;
	static float linex			= 0;
	int width, height; canvas->GetSize(&width, &height);

	// allocate the images to be used
	if (m_images.Len() != g_globals.dataset->dims + NUM_DERIVED_DIMS ||
		m_images[0]->Width() != size)
	{
		size = (int) (sqrt(static_cast<float>(m_NumClusters)) + 0.5f) + 1;
		if (hier_info->SHOW_BANDS)
			imgsize = (int) (sqrt(static_cast<float>(g_globals.dataset->data_size)) + 0.5f) + 1;
		else
			imgsize = size;

		// delete things before allocating new ones
		for (int i = 0; i < m_images.Len(); ++i)
		{
			delete (Image *) m_images[i];
			delete (Subwindow *) m_subwindows[i];
			m_images[i] = NULL; m_subwindows[i] = NULL;
		}
		m_images.Clear(); m_subwindows.Clear();
		m_images.SetLen(g_globals.dataset->dims + 1);
		m_subwindows.SetLen(g_globals.dataset->dims + 1);
		canvas->boundingBox.Invalidate();

		// allocate new ones
		// figure out the number of subwindows that would fit on each row
		int numSubWindInRow;
		for (numSubWindInRow = 1; /*empty*/ ; ++numSubWindInRow)
		{
			if (numSubWindInRow * numSubWindInRow >= g_globals.dataset->dims)
				break;
		}

		float xsize = imgsize / (float) width, ysize = imgsize / (float) height;
		int k	= -1;

		for (int dim = 0;
			 dim < g_globals.dataset->dims + NUM_DERIVED_DIMS;
			 ++dim)
		{
			Image *img	= new Image(imgsize + 1, 
									imgsize + 1);
			m_images[dim] = img;

			// figure out where to place the subwindow
			const float xs	= (xsize + 5.0f / width);
			float x = 0, y = 0;
			if (dim > 0)
			{
				x = (((dim - 1) % numSubWindInRow) * xs) + xs + 0.04f;
				y = ((((dim - 1) % numSubWindInRow == 0) ? ++k : k) * (ysize +
																	   18.0f /
																	   height));
			}

			if (dim == 0)
				linex = xsize + 0.02f;

			//			const float xs = (xsize + 5.0f/width);
			//			float x = 0, y = 0;
			////			if (dim < NUM_DERIVED_DIMS) {
			////				if (dim == 1)
			////					y = (ysize + 18.0f/height);
			////
			////			} else {
			//				x = (((dim-NUM_DERIVED_DIMS) % numSubWindInRow) * xs) + xs;
			//				y = ((((dim-NUM_DERIVED_DIMS) % numSubWindInRow == 0) ? ++k : k) 
			//					* (ysize + 18.0f/height));
			////			}
			Subwindow *subwnd	= new Subwindow(dim);
			subwnd->SetWindow(x, y, xsize, ysize);
			m_subwindows[dim] = subwnd;

			canvas->boundingBox.Add(subwnd->GetBoundingBox());
		}
	}

	if (m_zoomDim)
	{
		Subwindow *subwnd	= m_subwindows[m_currDim];
		Vec2 translate		(subwnd->GetX(), subwnd->GetY());
		//Vec2 zoom = Vec2(width/(size+1), height/(size+1));

		BBox2D swbbox		= subwnd->GetBoundingBox();
		swbbox.SetScale(1.0 * width / height);
		Vec2 zoom	(1.0 / swbbox.Width(), 1.0 / swbbox.Height());

		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		glScalef((float) zoom.X, (float) zoom.Y, 1.0f);
		glTranslatef((float) - translate.X, (float) - translate.Y, 1.0f);

		if (m_currDim < NUM_DERIVED_DIMS)
			subwnd->PaintClusters(m_images[m_currDim], size, hier_query, hier_info, true);
		else
			subwnd->PaintClusters(m_images[m_currDim], size, hier_query, hier_info, true);
	} else
	{
		//		if (m_autoZoom) 
		//		{
		////			Vec2 translate(m_BBox.Lowx(), m_BBox.Lowy());
		////			Vec2 zoom = Vec2(width/(m_BBox.Width()*size+1), height/(m_BBox.Height()*size+1));
		//			BBox2D bbox = m_BBox;
		//			bbox.SetScale(1.0*width/height);
		//			Vec2 zoom(1.0/bbox.Width(), 1.0/bbox.Height());
		//
		////			glMatrixMode(GL_MODELVIEW);
		////			glLoadIdentity();
		//			glScalef((float) zoom.X, (float) zoom.Y, 1.0f);
		////			glTranslatef((float) -translate.X, (float) -translate.Y, 1.0f);
		//		}

		Vec2 font_size	= canvas->GetDataFontSize("W");

		// for all subwindows
		for (int dim = 0;
			 dim < g_globals.dataset->dims + NUM_DERIVED_DIMS;
			 ++dim)
		{
			Subwindow *subwnd	= m_subwindows[dim];

			double max_str_size	= subwnd->GetWidth() - font_size.X;

			RGBt color	(interface_color[TEXT]);
			glColor3f((float) color.R, (float) color.G, (float) color.B);
			// paint the subwindow
			if (dim < NUM_DERIVED_DIMS)
			{
				if (dim == 0)
					canvas->DrawConfinedString(subwnd->GetX(),
											   subwnd->GetY() - (7.0f / width),
											   "Combined Distance", max_str_size);
				glLineWidth(2.0f);
				canvas->DrawLine(linex, 1.04f, linex, -.04f);
				glLineWidth(1.0f);
				//				else
				//					canvas->DrawConfinedString(subwnd->GetX(), subwnd->GetY()-(7.0f/width), 
				//						"Unbrushed", max_str_size);
				subwnd->PaintClusters(m_images[dim], size, hier_query, hier_info, dim == m_currDim);
			} else
			{
				canvas->DrawConfinedString(subwnd->GetX(),
										   subwnd->GetY() -
										   (7.0f / width),
										   g_globals.dataset->names[dim - NUM_DERIVED_DIMS],
										   max_str_size);
				subwnd->PaintClusters(m_images[dim], size, hier_query, hier_info, dim == m_currDim);
			}
		}
	}

	m_currCluster = 0;
	hier_combined_distances.Clear();
	hier_combined_distances.SetLen(m_NumClusters);
	for (int i = 0; i < m_NumClusters; ++i)
		hier_combined_distances[i] = 0;
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CHierPixel::DrawSingleCluster(const Cluster &c, bool brushed)
{
	if (hier_info->SHOW_BANDS)
		hier_query.sizes[m_currCluster] = c.entries;
	else
		hier_query.sizes[m_currCluster] = 1;
	//	m_clusters[m_currCluster++].Set(c, brushed);
	for (int j = 0; j < g_globals.dataset->dims; ++j)
	{
		// get the proper index of the data item that we want
		double val	= c.mean[j];

		// distance from the brush center to the data
		int idx		= m_currCluster * (g_globals.dataset->dims + 1) + (j + 1);
#ifdef _DEBUG
		double d = g_globals.curr_brush->Pos(j);
		double v = val - d;
		if (v<-1e50) {
			printf ("v=%f\n", v);
		}
#endif
		hier_query.distances[idx] = val - g_globals.curr_brush->Pos(j);

		// add all the normalized distances for this data point
		hier_combined_distances[m_currCluster] += g_globals.dataset->weight[j] * hier_query.distances[idx];

		//			printf("%d    %.2f   %.2f    %.2f\n", j+1, val, hierBrush.Min(j), hierBrush.Max(j));
	} // for (int j=0; j<all_dims; ++j)

	++m_currCluster;
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CHierPixel::InputMouseMotion(int sx, int sy)
{
	if (!g_globals.dataset)
		return;

	ShowCaption(canvas, sx, sy);

	if (canvas->button[2] == true)
		appCanvasMousePan(canvas);
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CHierPixel::InputButtonPress(int button, int x, int y)
{
	if (!g_globals.dataset)
		return;

	if (button == 2)
	{
		canvas->SetHandCursor();
		m_isPanning = true;
		return;
	}

	Vec2 p		= canvas->MapScreenToData(Vec2(x, y));
	int nearDim	= FindNearestDim(p);

	if (button == 0)
	{
		if (nearDim != -1)
		{
			m_currDim = nearDim;
			if (canvas->controlkey)
			{
				if (m_sortedDim == nearDim)
					m_isAscending = !m_isAscending;
				else
					m_sortedDim = nearDim;
			}
			appRefreshAllCanvas(Togl_Interp(canvas->GetWidget()));
		}
	} else if (button == 1)
	{
		if (m_currDim == nearDim)
		{
			m_zoomDim = !m_zoomDim;
			appRefreshAllCanvas(Togl_Interp(canvas->GetWidget()));
		}
	}
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
int CHierPixel::FindNearestDim(Vec2 p)
{
	if (!g_globals.dataset)
		return -1;
	for (int i = 0; i < g_globals.dataset->dims + 1; ++i)
	{
		int n	= 0; 
		if (m_subwindows[i]->Contains((float) p.X, (float) p.Y, n))
			return i;
	}
	return -1;
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CHierPixel::InputButtonRelease(int button, int x, int y)
{
	if (canvas->shiftkey)
		return;

	if (button == 2)
		m_isPanning = false;

	canvas->SetDefaultCursor();
	appRefreshAllCanvas(Togl_Interp(canvas->GetWidget()));
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CHierPixel::ShowCaption(grCanvas *canvas, int x, int y)
{
	Vec2 p	= canvas->MapScreenToData(Vec2(x, y));

	ostringstream oss;
	for (int i = 0; i < g_globals.dataset->dims + NUM_DERIVED_DIMS; ++i)
	{
		if (m_subwindows[i]->Contains((float) p.X, (float) p.Y))
		{
			if (i == 0)
				oss << "Combined Distance";
			else
				oss << "Dimension: " << g_globals.dataset->names[i - 1];
			break;
		}
	}

	appShowCanvasMessage(Togl_Interp(canvas->GetWidget()), oss.str().c_str(),
						 canvas->name);
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Draw coordinate axes name and min max.
void CHierPixel::DrawAxesNameMinMax(grCanvas *canvas)
{
	glShadeModel(GL_SMOOTH);
	canvas->SetForeground(interface_color[TEXT]);

	Vec2 font_size	= canvas->GetDataFontSize("W");

	double max_str_size, x = 0.985f;
	if (m_currDim == 0)
	{
		max_str_size = g_globals.screen_distortion->paracoord_interval[g_globals.dataset->dims - 1]; 
		canvas->DrawConfinedString(x, font_size.Y, "Combined Distance",
								   max_str_size);
	} else
	{
		max_str_size = g_globals.screen_distortion->paracoord_interval[m_currDim - 1] -
					   font_size.X * 5; 
		canvas->DrawConfinedString(x, font_size.Y,
								   g_globals.dataset->names[m_currDim - 1],
								   max_str_size);
		// Draw the MAX2 values of the dimension
		canvas->DrawConfinedString(x, -.51 * font_size.Y,
								   g_globals.dataset->GetMax(m_currDim - 1),
								   max_str_size);
		// Draw the MIN2 values of the dimension
		canvas->DrawConfinedString(x, 1.01f - font_size.Y,
								   g_globals.dataset->GetMin(m_currDim - 1),
								   max_str_size);
	}
}
//-----------------------------------------------------------------------------


//void CHierPixel::DrawAllClusters()
//{
//	//if both means and bands of the clusters need not to be shown, return;
//	if ( ! ( hier_info->SHOW_MEANS || hier_info->SHOW_BANDS ) )
//		return;
//
//	TraceDrawAllClusters( g_globals.btree->nodes[0] );
//}
//
//void CHierPixel::TraceDrawAllClusters(const Cluster &node)
//{
//	// if brushed cluster
//	if ( BITTEST(node.flags,IS_CONTOUR1) )
//	{
////		if ( ! BITTEST(node.flags,IS_BRUSHED_ANY) )
////			return;
//
//		if ( BITTEST(node.flags,IS_BRUSHED) ) {
//			DrawSingleCluster1(node, true);
////			return;
//		}
////		else 
////			return;
//	}
//
//	// else if unbrushed cluster
//	if ( BITTEST(node.flags,IS_CONTOUR0) )
//	{
////		if ( BrushSemantic == 0 )
////		{
////			if ( BITTEST(node.flags,IS_BRUSHED) )
////				return;
////		}
//		if ( hier_info->database )
//			return;
//
//		if ( ! BITTEST(node.flags,IS_BRUSHED) ) {
//			DrawSingleCluster1(node, false);
////			return;
//		}
////		else 
////			return;
//	}
//
//	// else traverse more
////	else
//	{
//		for (int j=0; j<node.num_children; j++) 
//			TraceDrawAllClusters(g_globals.btree->nodes[node.child[j]]);
//	}
//}

//-----------------------------------------------------------------------------
void CHierPixel::ComputeQueryDist(int sortedDim /*= 0*/,
								  bool isAscending /*= true*/)
{
	for (int i = 0; i < m_NumClusters; ++i)
	{
		// combined distance
		double val	= hier_query.distances[i*(g_globals.dataset->dims + 1)] = fabs(hier_combined_distances[i]);

		// get the min and max relevance factors for normalization later on
		if (i == 0)
			hier_query.dist_min = hier_query.dist_max = val;
		else
		{
			if (val < hier_query.dist_min)
				hier_query.dist_min = val;
			if (val > hier_query.dist_max)
				hier_query.dist_max = val;
		}
	}

	hier_query.brush_level = ((hier_query.dist_max - hier_query.dist_min) /
												 (m_NumClusters)) * m_NumBrushedClusters;

	// sort the distances
	g_globals.dataset->SortDist(&hier_query, sortedDim,
								isAscending);
}
//----------------------------------------------------------------------------- 

void CHierPixel::CountBrushedClusters(const Cluster &node)
{
	if (!BITTEST(node.flags, IS_BRUSHED_ANY))
		return;

	if (BITTEST(node.flags, IS_CONTOUR1))
	{
		if (BITTEST(node.flags, IS_BRUSHED))
			++m_NumBrushedClusters;
		else
			return;
	} else
		for (int j = 0; j < node.num_children; j++)
			CountBrushedClusters(g_globals.btree->nodes[node.child[j]]);
}


void CHierPixel::CountUnbrushedClusters(const Cluster &node)
{
	if (BrushSemantic == 0)
	{
		if (BITTEST(node.flags, IS_BRUSHED))
			return;
	}

	if (BITTEST(node.flags, IS_CONTOUR0))
	{
		if (!BITTEST(node.flags, IS_BRUSHED))
			++m_NumUnbrushedClusters;
		else
			return;
	} else
		for (int j = 0; j < node.num_children; j++)
			CountUnbrushedClusters(g_globals.btree->nodes[node.child[j]]);
}

void CHierPixel::SetParameters()
{
	m_NumBrushedClusters = m_NumUnbrushedClusters = 0;
	CountBrushedClusters(g_globals.btree->nodes[0]);
	CountUnbrushedClusters(g_globals.btree->nodes[0]);

	m_NumClusters = m_NumBrushedClusters + m_NumUnbrushedClusters;
	m_currCluster = 0;

	hier_query.data_size = m_NumClusters;
	hier_query.distances.SetLen((g_globals.dataset->dims + 1) * m_NumClusters);
	hier_query.randomNums.SetLen(m_NumClusters);
	hier_query.sizes.SetLen(m_NumClusters);
	hier_combined_distances.SetLen(m_NumClusters);
	for (int i = 0; i < m_NumClusters; ++i) {
		hier_combined_distances[i] = 0;
		hier_query.randomNums[i] = g_globals.GetRandomNum();
	}

	hierModifyBrushParametersByExtents();

	CHierDisplay::SetParameters();
}

//Added by Zaixian start
void CHierPixel::SetSortedDim(int dim) 
{
	if (m_sortedDim == dim)
		return;
	m_sortedDim = dim;
	//m_isBrushModified = true;
	canvas->Refresh();
}

void CHierPixel::SetSortDimOrder(bool isAscending)
{

	if (m_isAscending == isAscending)
		return;

	m_isAscending = isAscending;
	//m_isBrushModified = true;
	canvas->Refresh();
	
}

void CHierPixel::SelectDim(int dim) 
{
	if (m_currDim == dim)
		return; 

	m_currDim = dim;
	canvas->Refresh();
}

void CHierPixel::SetDimDistortion(int dim, float dist) 
{
	/*
	assert(dist >= 0 && dist <= 1.0f);
	if (m_distortions[dim] == dist)
		return;

	m_hasDistortionChanged = true;
	m_distortions[dim] = dist;
	canvas->Refresh();
	*/
}

void CHierPixel::ShowColormap(bool bShow) 
{
	m_isShowingColormap = bShow;
	canvas->Refresh();
}

void CHierPixel::ShowDerived(bool bShow) 
{
	
	m_showingDerived = (bShow == true) ? 0 : 1;
	if (m_currDim == 0 && m_showingDerived == 1)
		SelectDim(1);
	canvas->Refresh();
	
}

void CHierPixel::SetPlacementType(PlacementType pt) 
{
	/*
	if (m_placementType == pt)
		return;

	m_placementType = pt;
	switch (pt)
	{
	case PPT_NORMAL:
		if (m_dispPos)
		{
			delete[] m_dispPos[0];
			delete[] m_dispPos[1];
			delete[] m_dispPos;
			m_dispPos = NULL;
		}
		break;
	case PPT_MDS:
		if (!m_dispPos)
		{
			m_dispPos = new double * [2];
			m_dispPos[0] = new double [g_globals.dataset->dims + 1];
			m_dispPos[1] = new double [g_globals.dataset->dims + 1];
		}
		break;
	default:
		assert(true);
	}

	m_isBrushModified = true;
	hasPlacementChanged = true;
	canvas->Refresh();
	*/
}

void CHierPixel::SetSamplingFactor(float val) 
{
	/*
	assert(val >= 0 && val <= 1.0f);
	m_sampleFactor = val;
	m_sampleStart = g_globals.GetRandomNum() * (1-m_sampleFactor);
	m_sampleEnd = m_sampleStart + m_sampleFactor;
	reset = true;
	canvas->Refresh();
	*/
}

float CHierPixel::GetDimDistortion(int dim) const
{
	//return m_distortions[dim];
	return 1.0;
}

int CHierPixel::GetSelectedDim() const
{
	return m_currDim;
}
//Zaixian end
