//modified by Jing Yang after 7/6/2001
#include <xmdv/InterRing/CInterRingClusterTree.h>
#include <stdio.h>
#include <xmdv/XmdvTool.h>
#include <string.h>

CInterRingClusterTree::CInterRingClusterTree()
{
//	dim_cluster_file = NULL;
	CMAP = new colormap("InterRing");
	current_list = NULL;
    root = NULL;
}


CInterRingClusterTree::~CInterRingClusterTree()
{
	delete CMAP;
    if ( root != NULL )
        delete root;
}

//construct the cluster tree for interRing display according to the arrays;
//parameters: int num_nodes-number of nodes.
//parameters: array nodes-nodes stored in an array according to their ids. nodes[0] must be the root node.
//parameters: array parent_ids-corresponding nodes' parent' ids. nodes[0](the root node)'s parent id is 
//-1. Other parent ids should be larger than -1.
bool CInterRingClusterTree::ConstructInterRingClusterTree(int num_nodes, ARR<CInterRingCluster *> nodes, ARR<int> parent_ids)
{
	//if can't find the root node, return false
	if ( parent_ids[0] != -1 || nodes[0] == NULL)	
		return false;
	
	//link the clusters into a tree
	root = nodes[0];
	for ( int i = 1; i < num_nodes; i++ )
	{
		if ( parent_ids[i] < 0 || parent_ids[i] >= num_nodes )
			return false;
		if ( nodes[parent_ids[i]] == NULL )
			return false;
		nodes[parent_ids[i]]->AddChild(nodes[i]);
	}

	ResetLevels();
	ResetAngles();
	ResetColors();
	return true;
}

//set level for every node in the tree, find the max_level of the tree
void CInterRingClusterTree::ResetLevels()
{
	max_level = 1;
	root->ResetLevels(1, max_level);
}

void CInterRingClusterTree::ResetEntries()
{
    root->ResetEntries();
}


//reset color of all nodes of the tree. color ~ mid angle of the node
void CInterRingClusterTree::ResetColors()
{
	root->ResetColors(*CMAP);
}

//reorder
void CInterRingClusterTree::Reorder()
{
    if ( root->SetReorderPara() )
	    root->Reorder();
}


//set start_angles and sweep_angles for all the nodes
void CInterRingClusterTree::ResetAngles()
{
	unit_angle = 360. / root->entries;
	mini_angle = unit_angle / 25;
	root->ResetAngles(0, unit_angle);
}


CInterRingCluster* CInterRingClusterTree::FindNodeAccordingLevelAndAngle(int level, double angle)
{
	if ( level < root->level )
		return NULL;

	while ( angle < 0 )
		angle += 360;
	while ( angle > 360 )
		angle -= 360;

	return root->FindNodeAccordingLevelAndAngle(level, angle);
}


CInterRingCluster* CInterRingClusterTree::FindNodeAccordingLevelAndAngle(int level, double angle, bool &start, bool &end, bool &in)
{
	if ( level < root->level )
		return NULL;

	while ( angle < 0 )
		angle += 360;
	while ( angle > 360 )
		angle -= 360;

	start = false;
	end = false;
	in = false;

	return root->FindNodeAccordingLevelAndAngle(level, angle, mini_angle/2, start, end, in);
}


void CInterRingClusterTree::ResetCurrentList()
{
	if ( current_list != NULL )
		delete current_list;

	current_list = new CLinkList();
	root->ResetCurrentList(current_list);
}

void CInterRingClusterTree::ResetSelection()
{
}


void CInterRingClusterTree::Rotate(double degree)
{
	root->Rotate(degree);
}

//return double from 0 to 1;
double CInterRingClusterTree::GetDistance(CInterRingCluster *node_1, CInterRingCluster *node_2)
{
    CInterRingCluster *node1, *node2;

    if ( node_1->level >= node_2->level )
    {
        node1 = node_1;
        node2 = node_2;
    }
    else
    {
        node1 = node_2;
        node2 = node_1;
    }

//
    double distort_para1 = node1->sweep_angle / node1->entries / unit_angle;
    double distort_para2 = node2->sweep_angle / node2->entries / unit_angle;
    
    double distort_para = distort_para1 > distort_para2 ? distort_para1 : distort_para2;
//

    int dis = node1->level - node2->level;

    for ( int i = 0; i < dis; i++ )
    {
        node1 = node1->parent;
    }

    while ( node1 != node2 )
    {
        node1 = node1->parent;
        node2 = node2->parent;
        dis++;
    }
    double distance = node1->radius * distort_para;
//    double distance = ((double)dis)/(max_level-1);
    return distance;
}

