///////////////////////////////////////////////////////////// COPYRIGHT //
//
//  XmdvTool - A program for Multivariate Data Display
//
//  Copyright 1994-2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
///////////////////////////////////////////////////////////// COPYRIGHT //

//////////////////////////////////////////////////////////////////////////
/// \file ResultSet.cpp
/// \brief [Brief description about contents of the file]
/// 
/// \author 
//////////////////////////////////////////////////////////////////////////

/*============================== INCLUDES ==============================*/

#include <DataBaseLibrary/ResultSet.h>	// Short reason about y this file is included
#include<string>
#include<iostream>
#include<stdio.h>
#include<DataBaseLibrary/DataTypeNotSupportedException.h>


/*========================== PUBLIC FUNCTIONS ==========================*/
/// \name Structors
//@{
/// Constructor
ResultSet ::ResultSet(otl_stream *otlStream)
{
	this->otlStream = otlStream;
	this->row = NULL;
	this->numOfColumns = this->makeDataValue(*otlStream,&row);
}



/**
 * Destructor 
 * Destroys all the memory allocated by the result set.
 * Also it closes the otl_stream and deletes the stream.
 * \return 
 */
ResultSet :: ~ResultSet()
{
	for(int i=0;i<this->numOfColumns;i++)	
		delete(this->row[i]);
	delete[] row;	
	this->otlStream->close();
	//Removing the stream
	delete(this->otlStream);
}

//@}
/** Returns the current row number.
* @return The current row number in the result set, or <code>0</code>
* if it can't be determined.
*/
int ResultSet :: getRow()
{
	return(this->otlStream->get_rpc());
}

/** Moves to the next row in the result set 
* @return true if the cursor is in the result set
*/
bool ResultSet :: next()
{
	bool b = false;	
	if(!this->otlStream->eof())
	{
		//Stores the row in the array of data values.
		for(int i=0;i< this->numOfColumns;i++)
		{
			this->row[i]->setValue(*this->otlStream);
		}
		b = true;
	}
	return(b);
}


/** Gets a column's value as a double
* @param idx The column index, starting at 1
*/
double ResultSet :: getDouble(int idx)
{
	double temp = *(double*)this->row[idx]->getValue();
	return(temp);	
}



/** Gets a column's value as a float
* @param idx The column index, starting at 1
*/
float ResultSet :: getFloat(int idx)
{	
	//This a hack because the desc_select does not work prefect
	//int temp = *(int*)this->row[idx]->getValue();
	//float temp = *(float*)this->row[idx]->getValue();
	float temp = (float) this->getDouble(idx);
	return(temp);	
}

/** Gets a column's value as an int
* @param idx The column index, starting at 1
*/
int ResultSet :: getInt(int idx)
{
	//This a hack becuase the desc_select does not work prefect
	//int temp = *(int*)this->row[idx]->getValue();
	int temp = (int) this->getLong(idx);
	return(temp);
}

/** Gets a column's value as a Long
* @param idx The column index, starting at 1
*/
long ResultSet :: getLong(int idx)
{
	long temp = *(long*)this->row[idx]->getValue();
	return(temp);
}


/** Gets a column's value as a string
* @param idx The column index, starting at 1
*/
std::string ResultSet :: getString(int idx)
{
	std::string temp = *(std::string*)this->row[idx]->getValue();
	return(temp);
}

/*
* This function returns a pointer of the otl_column_desc 
* type to the descriptor of the output column list: 
* The pointer points to an internal structure inside the stream which 
* gets deallocated at the moment of the stream desctruction, so the user does not need to do any memory deallocation operations with the pointer. 
* @The desc_len returns the lenght of the allocated array.
*/
otl_column_desc* ResultSet ::describe_select(int& desc_len)
{
	return(this->otlStream->describe_select(desc_len));
}

/**
 * Returns the number of columns in the resultSet.
 * \return numOfColumns
 */
int ResultSet :: getNumberOfColumns()
{
	return(this->numOfColumns);
}

/**
 * Returns true if the result set is empty.
 * \return 
 */
bool ResultSet:: isDone()
{
	if(this->otlStream->eof())
		return(true);
	else
		return(false);	
}
/**
 *
 * \param this->row 
 */
int ResultSet ::makeDataValue(otl_stream &inputStream,DataValue ***row)

{	int numberOfColumns;
	//This function sets the number of columns
	const otl_column_desc* columnDesc = this->describe_select(numberOfColumns);	

	enum DataValue::Type t;

	*row = new DataValue*[numberOfColumns];

	for(int i=0;i<numberOfColumns;i++)
	{
		int type = columnDesc[i].otl_var_dbtype;
		switch(type)
		{
			case 1://String
				t = DataValue::str;
				(*row)[i]	= new DataValue(t,columnDesc[i].dbsize);
				break;
			case 4:		//int
				t = DataValue::integer;
				(*row)[i]	= new DataValue(t,columnDesc[i].dbsize);
				break;
			case 7:		//long
				t = DataValue::lInteger;
				(*row)[i]	= new DataValue(t,columnDesc[i].dbsize);
				break;
			case 2:		//Double								
				//Changing the type to int
				if(columnDesc[i].scale <= 0 && columnDesc[i].prec != 38)//The prec accounts for float type
				{
					t = DataValue::lInteger;
					this->otlStream->set_column_type(i,otl_var_int);
				} // if(columnDesc[i].scale <= 0)
				else 				
					t= DataValue::lReal;
				(*row)[i]	= new DataValue(t,columnDesc[i].dbsize);				
				break;

			case 3:		//Float
				t = DataValue::real;
				(*row)[i]	= new DataValue(t,columnDesc[i].dbsize);
				break;				
			default:				
				{
				//Recover from exception
				this->numOfColumns = 0;
				for(int j=0;j<i;j++)
					delete((*row)[i]);				
				DataTypeNotSupportedException exception;
				throw(exception);				
				}
		} // switch(type)	
		
	} // for(int i=0;i<numberOfColumns;i++)

	return(numberOfColumns);
}


