#include<backend/PreparedStatement.h>
#include<backend/ResultSet.h>
#include<backend/DataTypeNotSupportedException.h>
/**
*
* \date 02-20-2004
*
* \author
*This class gives an interface similar to the prepared statement in Java
*/
	/**
	* Constructs a prepared statement given the name
	* \param sqlStatement 
	* \param otlConnectPtr 
	* @post If the buffer is greater than one make sure that
	* the destructor of the object gets called, because that 
	* will flush the buffer
	* \return 
	*/
PreparedStatement::PreparedStatement(std::string sqlStatement,
									 otl_connect* otlConnectPtr,
									 const unsigned int bufferSize)
{				
	this->stream = new otl_stream (bufferSize, // buffer size
		sqlStatement.c_str(), //SQLStatement							
		*otlConnectPtr// connect object
		);		
	int desc_len;
	otl_var_desc* column_desc = this->stream->describe_in_vars(desc_len);
	this->length = desc_len;
	this->arrayVariables = new void *[desc_len];
	for(int i=0;i<desc_len;i++)
	{
		vecType.push_back(column_desc[i].ftype);
		//Allocate memory to the void pointers
		//Except for the var_char because their length is not known now.

		switch(column_desc[i].ftype)
		{

		case 1://string
			this->arrayVariables[i] = new char[1];
			break;

		case 2://Double
			this->arrayVariables[i] = new double;
			break;

		case 3://Float
			this->arrayVariables[i] = new float;
			break;

		case 4: //Integer
			this->arrayVariables[i] = new int;
			break;

		case 7://Long
			this->arrayVariables[i] = new long;
			break;	
		default:
			{	
				this->recover(i);
				DataTypeNotSupportedException exception;
				throw(exception);		
			}
		} // switch(fType)

	}

}

	/**
	* Note you need to call this 
	* Destorys a new prepared Statement
	* \return 
	*/
	PreparedStatement::~PreparedStatement()
	{
		for(int i=0;i<this->length;i++)
		{
			if(this->vecType[i] == 1) //If the type is a string
				delete [] this->arrayVariables[i];
			else
				delete(this->arrayVariables[i]);
		}
		delete[] (this->arrayVariables);
		//closing the input Stream	//Flushes the stream to the data base.
		this->stream->close();
	}

	/**
	* This method is used to recover from an error
	* \param i 
	* \return 
	*/
	void PreparedStatement ::recover(int i)
	{
		for(int j=0;j<i;j++)
		{
			if(this->vecType[i] == 1) //If the type is a string
				delete [] this->arrayVariables[i];
			else
				delete(this->arrayVariables[i]);
		} // for(int j=0;j<i;j++)		
		this->length = 0;//Disables the destructor
	}
	
	/**
	* Executes the prepared statement which must be an sql update
	* int the form of INSERT, DELETE and UPDATE
	*@pre the set on each and every column should be called
	*/
	void PreparedStatement ::executeUpdate()
	{
		execute();
	}

	/**
	* Executes the prepared statement which must be an sql update
	* int the form of INSERT, DELETE and UPDATE
	*@pre the set on each and every column should be called
	*@post The caller should delete the object once it has been used.
	*@return ResultSet
	*/
	ResultSet* PreparedStatement:: executeQuery()
	{
		execute();		
		return(new ResultSet(stream));
	}

	/**
	* This method uses the arrayVaraibles and the vec type to 
	* bind the actual values in place of place holders in the query.
	* and executes the sql_statement
	*/
	void PreparedStatement ::execute()
	{
		for(int i=0;i<length;i++)
		{		
			int fType = vecType[i];
			switch(fType)
			{

			case 1://string
				*(this->stream)<<(char*)this->arrayVariables[i];
				break;

			case 2://Double
				*(this->stream)<<*(double*)this->arrayVariables[i];
				break;

			case 3://Float
				*(this->stream)<<*(float*)this->arrayVariables[i];
				break;

			case 4: //Integer
				*(this->stream)<<*(int*)this->arrayVariables[i];
				break;

			case 7://Long
				*(this->stream)<<*(long*)this->arrayVariables[i];
				break;	

			} // switch(fType)

		} // for(int i=0;i<length;i++)	
	}
	/**
	* Sets the float value at the column index
	* \param item 
	* \param index 
	* \return 
	*/
	void PreparedStatement::setFloat(float item, int index)
	{
		*((float*) this->arrayVariables[index]) = item;
	}

	/**
	* Set the integer value at the column index
	* \param item 
	* \param index 
	* \return 
	*/
	void PreparedStatement::setInt(int item,int index)
	{
		*((int*) this->arrayVariables[index]) = item;
	}

	/**
	* Sets the double value at the column index
	* \param item 
	* \param index 
	* \return 
	*/
	void PreparedStatement ::setDouble(double item,int index)
	{
		*((double*) this->arrayVariables[index]) = item;
	}

	/**
	* Sets the string at the index
	* \param str 
	* \param index 
	* \return 
	*/
	void PreparedStatement ::setString(std::string str,int index)
	{
		delete[] this->arrayVariables[index];
		this->arrayVariables[index] = new char[str.length()+1];
		strcpy((char*) this->arrayVariables[index],str.c_str());
	}


	/**
	* This methods flushes the buffer to the data base.
	* To ensure that all the updates get reflected in the data base
	* call this method.	
	*/
	void PreparedStatement :: flush()
	{
		this->stream->flush();
	}

