#include <Brush.h>

using namespace cache;

/*
*	This stores the parameters of the current brush it is a singleton
*/
	/**
	* Constructs a new brush.
	* \return 
	*/
	Brush::Brush()
	{
		this->iShape = new Tools::Geometry::Region();
	}

	/**
	* Constructs a new brush
	* \param e1 
	* \param e2 
	* \param lod 
	* \param type 
	* \return 
	*/

	 Brush::Brush(double e1,double e2,double lod,int numberOfDimensions,int type)
	{	
		Type t;
		if(type == 0)
			t = Any;
		if(type == 1)
			t = All;
		this->e1 = e1;
		this->e2 = e2;
		this->lod = lod;
		this->brushType = t;		
		this->numberOfDimensions = numberOfDimensions;
		double low[2];
		double high[2];
		low[0] = e1;
		low[1] = lod;
		high[0] = e2;
		high[1] = lod;				
		this->iShape = new 
			Tools::Geometry::Region(
			low,high,
			numberOfDimensions);		
	}	

	/**
	* Constructs a new brush
	* \param e1 
	* \param e2 
	* \param lod 
	* \param brushType 
	* \return 
	*/
	 
	Brush::Brush(double e1,double e2,double lod,unsigned int numberOfDimensions,Type brushType)
	{
		this->e1 = e1;
		this->e2 = e2;
		this->lod = lod;
		this->brushType = brushType;		
		this->numberOfDimensions = numberOfDimensions;
		double low[2];
		double high[2];
		low[0] = e1;
		low[1] = lod;
		high[0] = e2;
		high[1] = lod;		
		this->iShape = new 
			Tools::Geometry::Region(low,high,
			numberOfDimensions);
	}

	/**
	* Define the copy constructor
	* \param &brush 
	* \return 
	*/
	Brush::Brush(Brush &brush)
	{
		this->e1 = brush.e1;
		this->e2 = brush.e2;
		this->lod = brush.lod;
		this->brushType = brush.brushType;
		this->numberOfDimensions = brush.numberOfDimensions;
		//Calling the = operator this allocates new memory
		this->iShape = new Tools::Geometry::Region(*(Tools::Geometry::Region*)
			brush.iShape);
	}

	Brush::Brush(int numberOfDimensions)
	{		
		this->iShape = NULL;
		this->numberOfDimensions  = numberOfDimensions;
	}
	
	bool Brush::operator ==(Brush &b)
	{
		if(this->e1 == b.e1 && this->e2 == b.e2 && this->lod == b.lod)
			return(true);
		else
			return(false);
	}

	Brush& Brush:: operator =(Brush &brush)
	{
		this->e1 = brush.e1;
		this->e2 = brush.e2;
		this->lod = brush.lod;
		this->brushType = brush.brushType;
		this->numberOfDimensions = brush.numberOfDimensions;
		//Calling the = operator this allocates new memory
		*(Tools::Geometry::Region*)(this->iShape) = *(Tools::Geometry::Region*)
			(brush.iShape);
		return(*this);
	}


	double Brush::getLeftExtent()
	{
		return(this->e1);
	}


	double Brush::getRightExtent()
	{
		return(this->e2);
	}

	double Brush::getLevelOfDetail()
	{
		return(this->lod);
	}

	/**
	* Sets the level of detail
	* \param lod 
	* \return 
	*/
	void Brush::setLevelOfDetail(double lod)
	{
		this->lod = lod;
	}

	/**
	* Returns the condition used to get the brush
	* \return 
	*/
	std::string Brush::getPredicate()
	{
		std::ostringstream ostring;
		if(brushType == Any)
			ostring<<" X <= "<<e2<<" and Y >= "<<e1<<" ";
		else
			ostring<<" X >= "<<e1<<" and Y <= "<<e2<<" ";

		return(ostring.str());
	}

	Brush::Type Brush:: getBrushType()
	{
		return(this->brushType);
	}

	//Checks to see if the value is really in the brush
	//Note if the lod of the current brush matches one of the level values
	//We will get 2 objects using intersect shape that is both the
	//parent and the child in the cluster tree. This method
	//checks for this condition and also the all brush.
	bool Brush:: inBrush(Tools::Geometry::IShape& iShape)
	{	
		bool result = false;
		//Stores the pointer to the region.
		Tools::Geometry::Region* regionPtr; 
		if(this->iShape->intersectsShape(iShape) == true)
		{			
			regionPtr = 
				dynamic_cast<Tools::Geometry::Region*> (&iShape);
			//getting the radius
			double L = regionPtr->getLow(1);
			//getting the parent radius of the object.
			double PL = regionPtr->getHigh(1);
				if(PL > this->lod && L <= this->lod)
					result = true;						
		}
		if(result && this->brushType == All)//If any brush we don't need to do this
		{			
			if(regionPtr->getLow(0) >= this->e1 && 
				regionPtr->getHigh(0) <= this->e2)		
				result = true;
		} // if(this->brushType = All)
		return(result);
	}

	Tools::Geometry::IShape& Brush::getShape()
	{
		return(*this->iShape);
	}

	/**
	* Deletes the iShape object
	* \return 
	*/
	Brush::~Brush()
	{
		delete(this->iShape);
	}

