#include <CacheRegion.h>
#include <ICacheEntry.h>
#include <RemainderRegion.h>

using namespace cache;

CacheRegion::CacheRegion(ICacheEntry &cacheEntry)
{	
	const Tools::Geometry::IShape *iShape = cacheEntry.getIShape();

	const Tools::Geometry::Region* regionPtr = 
		dynamic_cast<const Tools::Geometry::Region*>(iShape);

	this->leftExtent = 0;

	this->leftExtent = regionPtr->getLow(0);

	this->rightExtent = regionPtr->getHigh(0);

	this->levelOfDetail = regionPtr->getLow(1);		
}

//Copy Constructor
CacheRegion::CacheRegion(const CacheRegion &cacheRegion)
{
	this->leftExtent = cacheRegion.leftExtent;
	this->rightExtent = cacheRegion.rightExtent;
	this->levelOfDetail = cacheRegion.levelOfDetail;
}

//operator = undefined.
CacheRegion& CacheRegion::operator =(const CacheRegion &cacheRegion)
{
	this->leftExtent = cacheRegion.leftExtent;
	this->rightExtent = cacheRegion.rightExtent;
	this->levelOfDetail = cacheRegion.levelOfDetail;
	return(*this);
}

CacheRegion ::~CacheRegion()
{
	/*Note never call delete on regionPtr*/
}

bool CacheRegion:: operator < (CacheRegion& cacheRegion)
{	
	bool result = false;	
	//Comparing the X dimension
	if(this->leftExtent < cacheRegion.leftExtent)
		result = true;
	else 
		result = false;		
	return(result);
}

double CacheRegion:: getLeftExtent()
{
	return(this->leftExtent);
}

double CacheRegion:: getRightExtent()
{
	return(this->rightExtent);
}

/*
 *	Note here cache is simply a name space
 */
RemainderRegion* cache::fillGap(
						 CacheRegion& leftRegion,
						 CacheRegion& rightRegion,
						 double levelOfDetail )//Brush level of detail
{
	RemainderRegion *remainderRegion = NULL;
	if(leftRegion.rightExtent < rightRegion.leftExtent)
	{
		//Note all the inbetween remainder regions have to be
		// ALL
		remainderRegion = 
			new AllRemainderRegion(leftRegion.rightExtent,
			rightRegion.leftExtent,
			levelOfDetail);

	} // if(leftRegion.rightExtent < rightRegion.leftExtent)

	return(remainderRegion);
}

bool cache::compare (const CacheRegion& a, const CacheRegion& b){
	bool result = false;
	//Comparing the X dimension
	if(a.leftExtent < b.leftExtent)
		result = true;
	else
		result = false;
	return(result);

}

