#include <cache/DBFileParser.h>
#include <cache/Schema.h>
#include <string>
#include <cache/ConfigurationStore.h>
#include <cache/FileNotFoundException.h>
#include <fstream>
#include <mmgr/nommgr.h>

using namespace cache;
/**
* Constructs a DBFileParser 
* the parser also parses the configuration 
* file
* \param file 
* \return 
*/
DBFileParser::DBFileParser(std::string file)
{	
	//Checking to see if the file exists.
	std::ifstream inputStream(file.c_str(),std::ios::in);
	if(!inputStream.good())
		throw FileNotFoundException(file);
	inputStream.close();
	/////////////////////////////////////////////x

	this->schema = NULL;
	dbConfigStore = new DBConfigStore();
	SAX2DOM::Parser<std::string> domParser;
	SAX::CatchErrorHandler<std::string> eh;
	domParser.setErrorHandler(eh);	
	SAX::InputSource is;
	is.setSystemId(file);		
	domParser.parse(is);	
	DOM::Document<std::string> doc = domParser.getDocument();
	makeSchemaObject(doc);	
	
	dbConfigStore->tableName = this->tableName;	
	dbConfigStore->schema = this->schema;

	//Reading in the name of the contour table.
	DOM::NodeList<std::string> nodeList = doc.getElementsByTagName("ContourTable");
	DOM::Node<std::string> node = nodeList.item(0);
	DOM::NamedNodeMap<std::string> nodeMap = node.getAttributes();
	dbConfigStore->contourTable = nodeMap.getNamedItem("name").getNodeValue();

}

DBConfigStore* DBFileParser :: getConfigStore()
{
	return(this->dbConfigStore);
}

void DBFileParser::makeSchemaObject(DOM::Document<std::string> &doc)
{
	this->schema = new Schema();
	
	DOM::NodeList<std::string> nodeList = doc.getElementsByTagName("Table");
	DOM::Node<std::string> node = nodeList.item(0);
	DOM::NamedNodeMap<std::string> nodeMap = node.getAttributes();

	this->tableName = nodeMap.getNamedItem("name").getNodeValue();

	//	nodeMap.getNamedItem("numberOfTuples").getNodeValue();
	unsigned int numberOfDimensions = (unsigned int) atoi(nodeMap.getNamedItem("numberOfDimensions").getNodeValue().c_str());

	//Assigning the number of dimensions.
	this->schema->numberOfDimensions = numberOfDimensions;
	this->numberOfDimensions = numberOfDimensions;

	//Make all the meta columns
	//Makes all the data columns
	nodeList = doc.getElementsByTagName("MetaColumn");	

	node = nodeList.item(0);
	nodeMap = node.getAttributes();
	this->schema->columnShape.columnName = nodeMap.getNamedItem("shape").getNodeValue();

	node = nodeList.item(1);
	nodeMap = node.getAttributes();
	this->schema->columnX1.columnName = nodeMap.getNamedItem("x1").getNodeValue();

	node = nodeList.item(2);
	nodeMap = node.getAttributes();
	this->schema->columnX2.columnName = nodeMap.getNamedItem("x2").getNodeValue();

	node = nodeList.item(3);
	nodeMap = node.getAttributes();
	this->schema->columnY1.columnName = nodeMap.getNamedItem("y1").getNodeValue();

	node = nodeList.item(4);
	nodeMap = node.getAttributes();
	this->schema->columnY2.columnName = nodeMap.getNamedItem("y2").getNodeValue();

	//Reading the name for the "N" column.
	node = nodeList.item(5);
	nodeMap = node.getAttributes();
	this->schema->columnN.columnName = nodeMap.getNamedItem("N").getNodeValue();
		
	//Makes all the data columns
	nodeList = doc.getElementsByTagName("DataColumn");	
	
	for(unsigned int i =0;i<numberOfDimensions;i++)
	{
		DOM::Node<std::string> node = nodeList.item(i);

		DOM::NamedNodeMap<std::string> nodeMap = node.getAttributes();

		std::string columnName = nodeMap.getNamedItem("name").getNodeValue();

		this->schema->dataColumns.push_back(new Column(Column::data,columnName));		
	} // for(unsigned int i =0;i<numberOfDimensions;i++)

	
	//This is an important statement wrong design option
	this->schema->makeSelectString();
}

Schema* DBFileParser::getSchema()
{
	return(this->schema);
}

/**
 *
 * \return the name of the table
 */
std::string DBFileParser::getTableName()
{
	return(this->tableName);
}

DBFileParser::~DBFileParser()
{	}
